Package: DOSE
Type: Package
Title: Disease Ontology Semantic and Enrichment analysis
Version: 4.4.0
Authors@R: c( person(given = "Guangchuang", family = "Yu",        email = "guangchuangyu@gmail.com",   role = c("aut", "cre")),
              person(given = "Li-Gen",      family = "Wang",      email = "reeganwang020@gmail.com",   role = "ctb"),
              person(given = "Vladislav",   family = "Petyuk",    email = "petyuk@gmail.com",          role = "ctb"),
              person(given = "Giovanni",    family = "Dall'Olio", email = "giovanni.dallolio@upf.edu", role = "ctb"))
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Description: This package implements five methods proposed by
    Resnik, Schlicker, Jiang, Lin and Wang respectively
    for measuring semantic similarities among DO terms and
    gene products. Enrichment analyses including hypergeometric
    model and gene set enrichment analysis are also implemented
    for discovering disease associations of high-throughput
    biological data.
Depends: R (>= 3.5.0)
Imports: AnnotationDbi, BiocParallel, fgsea, ggplot2, GOSemSim (>=
        2.31.2), methods, qvalue, reshape2, stats, utils, yulab.utils
        (>= 0.1.6)
Suggests: prettydoc, clusterProfiler, gson (>= 0.0.5), knitr, memoise,
        org.Hs.eg.db, rmarkdown, testthat
VignetteBuilder: knitr
ByteCompile: true
License: Artistic-2.0
Encoding: UTF-8
URL: https://yulab-smu.top/contribution-knowledge-mining/
BugReports: https://github.com/GuangchuangYu/DOSE/issues
Packaged: 2025-10-30 03:42:04 UTC; biocbuild
biocViews: Annotation, Visualization, MultipleComparison,
        GeneSetEnrichment, Pathways, Software
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/DOSE
git_branch: RELEASE_3_22
git_last_commit: 963047a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Author: Guangchuang Yu [aut, cre],
  Li-Gen Wang [ctb],
  Vladislav Petyuk [ctb],
  Giovanni Dall'Olio [ctb]

# Generated by roxygen2: do not edit by hand

S3method("$",enrichResult)
S3method("$",gseaResult)
S3method("[",enrichResult)
S3method("[",gseaResult)
S3method("[[",enrichResult)
S3method("[[",gseaResult)
S3method(as.data.frame,enrichResult)
S3method(as.data.frame,gseaResult)
S3method(dim,enrichResult)
S3method(dim,gseaResult)
S3method(geneID,compareClusterResult)
S3method(geneID,enrichResult)
S3method(geneID,gseaResult)
S3method(geneInCategory,compareClusterResult)
S3method(geneInCategory,enrichResult)
S3method(geneInCategory,gseaResult)
S3method(head,enrichResult)
S3method(head,gseaResult)
S3method(tail,enrichResult)
S3method(tail,gseaResult)
export(EXTID2NAME)
export(clusterSim)
export(doSim)
export(doseSim)
export(enrichDGN)
export(enrichDGNv)
export(enrichDO)
export(enrichNCG)
export(facet_grid)
export(gene2DO)
export(geneID)
export(geneInCategory)
export(geneSim)
export(gseDGN)
export(gseDO)
export(gseNCG)
export(gsfilter)
export(mclusterSim)
export(parse_ratio)
export(setReadable)
export(simplot)
export(theme_dose)
exportClasses(compareClusterResult)
exportClasses(enrichResult)
exportClasses(gseaResult)
exportMethods(show)
exportMethods(summary)
importClassesFrom(GOSemSim,GOSemSimDATA)
importClassesFrom(methods,data.frame)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,multicoreWorkers)
importFrom(GOSemSim,combineScores)
importFrom(GOSemSim,load_OrgDb)
importFrom(GOSemSim,termSim)
importFrom(fgsea,fgsea)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(qvalue,qvalue)
importFrom(reshape2,melt)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(yulab.utils,yulab_msg)
importMethodsFrom(AnnotationDbi,columns)
importMethodsFrom(AnnotationDbi,exists)
importMethodsFrom(AnnotationDbi,get)
importMethodsFrom(AnnotationDbi,keys)
importMethodsFrom(AnnotationDbi,keytypes)
importMethodsFrom(AnnotationDbi,select)
importMethodsFrom(AnnotationDbi,toTable)

# DOSE: Disease Ontology Semantic and Enrichment analysis

[![](https://img.shields.io/badge/release%20version-3.30.2-green.svg)](https://www.bioconductor.org/packages/DOSE)
[![](https://img.shields.io/badge/devel%20version-3.31.3-green.svg)](https://github.com/guangchuangyu/DOSE)
[![Bioc](http://www.bioconductor.org/shields/years-in-bioc/DOSE.svg)](https://www.bioconductor.org/packages/devel/bioc/html/DOSE.html#since)
[![codecov](https://codecov.io/gh/GuangchuangYu/DOSE/branch/master/graph/badge.svg)](https://codecov.io/gh/GuangchuangYu/DOSE/)

[![Project Status: Active - The project has reached a stable, usable
state and is being actively
developed.](http://www.repostatus.org/badges/latest/active.svg)](http://www.repostatus.org/#active)
[![platform](http://www.bioconductor.org/shields/availability/devel/DOSE.svg)](https://www.bioconductor.org/packages/devel/bioc/html/DOSE.html#archives)
[![Build
Status](http://www.bioconductor.org/shields/build/devel/bioc/DOSE.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/DOSE/)
[![](https://img.shields.io/badge/download-836274/total-blue.svg)](https://bioconductor.org/packages/stats/bioc/DOSE)
[![](https://img.shields.io/badge/download-20017/month-blue.svg)](https://bioconductor.org/packages/stats/bioc/DOSE)

This package implements five methods proposed by Resnik, Schlicker,
Jiang, Lin and Wang respectively for measuring semantic similarities
among DO terms and gene products. Enrichment analyses including
hypergeometric model and gene set enrichment analysis are also
implemented for discovering disease associations of high-throughput
biological data.

## :writing_hand: Authors

Guangchuang YU <https://yulab-smu.top>

School of Basic Medical Sciences, Southern Medical University

Learn more at <https://yulab-smu.top/contribution-knowledge-mining/>.

Please cite the following article when using `DOSE`:

Guangchuang Yu, Li-Gen Wang, Guang-Rong Yan, Qing-Yu He. DOSE: an
R/Bioconductor package for Disease Ontology Semantic and Enrichment
analysis. Bioinformatics. 2015, 31(4):608-609.

##' Enrichment analysis based on the Network of Cancer Genes database (http://ncg.kcl.ac.uk/)
##'
##' given a vector of genes, this function will return the enrichment NCG
##' categories with FDR control
##'
##' 
##' @title enrichNCG
##' @param gene a vector of entrez gene id
##' @param pvalueCutoff pvalue cutoff
##' @param pAdjustMethod one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"
##' @param universe background genes
##' @param minGSSize minimal size of genes annotated by NCG category for testing
##' @param maxGSSize maximal size of each geneSet for analyzing
##' @param qvalueCutoff qvalue cutoff
##' @param readable whether mapping gene ID to gene Name
##' @return A \code{enrichResult} instance
##' @export
##' @author Guangchuang Yu
enrichNCG <- function(gene,
                      pvalueCutoff = 0.05,
                      pAdjustMethod = "BH",
                      universe,
                      minGSSize = 10,
                      maxGSSize = 500,
                      qvalueCutoff = 0.2,
                      readable = FALSE){
    
    enrichDisease(gene = gene,
                  pvalueCutoff = pvalueCutoff,
                  pAdjustMethod = pAdjustMethod,
                  universe = universe,
                  minGSSize = minGSSize,
                  maxGSSize = maxGSSize,
                  qvalueCutoff = qvalueCutoff,
                  readable = readable,
                  ontology = "NCG")
}

get_NCG_data <- function() {
    if (!exists(".DOSEenv")) .initial()
    .DOSEEnv <- get(".DOSEEnv", envir = .GlobalEnv)
    
    if (!exists(".NCG_DOSE_Env", envir=.DOSEEnv)) {
        tryCatch(utils::data(list="NCG_EXTID2PATHID", package="DOSE"))
        tryCatch(utils::data(list="NCG_PATHID2EXTID", package="DOSE"))
        tryCatch(utils::data(list="NCG_PATHID2NAME", package="DOSE"))
        EXTID2PATHID <- NCG_EXTID2PATHID <- get("NCG_EXTID2PATHID")
        PATHID2EXTID <- NCG_PATHID2EXTID <- get("NCG_PATHID2EXTID")
        PATHID2NAME <- NCG_PATHID2NAME <- get("NCG_PATHID2NAME")

        rm(NCG_EXTID2PATHID, envir = .GlobalEnv)
        rm(NCG_PATHID2EXTID, envir = .GlobalEnv)
        rm(NCG_PATHID2NAME, envir = .GlobalEnv)

        assign(".NCG_DOSE_Env", new.env(), envir = .DOSEEnv)
        .NCG_DOSE_Env <- get(".NCG_DOSE_Env", envir = .DOSEEnv)
        assign("EXTID2PATHID", EXTID2PATHID, envir = .NCG_DOSE_Env)
        assign("PATHID2EXTID", PATHID2EXTID, envir = .NCG_DOSE_Env)
        assign("PATHID2NAME", PATHID2NAME, envir = .NCG_DOSE_Env)
    }
    
    get(".NCG_DOSE_Env", envir = .DOSEEnv)
}

##' @method as.data.frame enrichResult
##' @export
as.data.frame.enrichResult <- function(x, ...) {
    x <- get_enriched(x)
    as.data.frame(x@result, ...)
}

##' @method as.data.frame gseaResult
##' @export
as.data.frame.gseaResult <- function(x, ...) {
    as.data.frame(x@result, ...)
}

##' @method geneID enrichResult
##' @export
geneID.enrichResult <- function(x) as.character(x@result$geneID)

##' @method geneID gseaResult
##' @export
geneID.gseaResult <- function(x) as.character(x@result$core_enrichment)

##' @method geneID compareClusterResult
##' @export
geneID.compareClusterResult <- function(x) as.character(x@compareClusterResult$geneID)


##' @method geneInCategory enrichResult
##' @export
##' @importFrom stats setNames
geneInCategory.enrichResult <- function(x)
    setNames(strsplit(geneID(x), "/", fixed=TRUE), rownames(x@result))

##' @method geneInCategory gseaResult
##' @export
geneInCategory.gseaResult <- function(x)
    setNames(strsplit(geneID(x), "/", fixed=TRUE), rownames(x@result))

##' @method geneInCategory compareClusterResult
##' @export
geneInCategory.compareClusterResult <- function(x) {
########## v1
    ## setNames(strsplit(geneID(x), "/", fixed=TRUE), 
    ## paste(x@compareClusterResult$Cluster, 
    ## x@compareClusterResult$ID, sep= "-"))
    ## setNames(strsplit(geneID(x), "/", fixed=TRUE), x@compareClusterResult$ID)

########## v2
    ## Cluster <- NULL
    ## clusters <- as.character(unique(x@compareClusterResult$Cluster))
    ## list_new <- setNames(lapply(clusters, 
    ##     function(j) dplyr::filter(x@compareClusterResult, Cluster == j)), clusters)
        
    ## for(i in seq_len(length(list_new))) {
    ##     list_new[[i]] <- setNames(strsplit(as.character(list_new[[i]]$geneID), 
    ##                                        "/", fixed=TRUE), 
    ##                               list_new[[i]]$ID)
    ## }
    ## return(list_new)


    x <- as.data.frame(x)
    # reslist <- split(x@compareClusterResult, x@compareClusterResult$Cluster)
    reslist <- split(x, x$Cluster)
    if ("core_enrichment" %in% colnames(x)) {
        res <- lapply(reslist, function(y)
            setNames(
                strsplit(y$core_enrichment, split="/", fixed=TRUE),
                y$ID
            ))
    } else {
        res <- lapply(reslist, function(y)
            setNames(
                strsplit(y$geneID, split="/", fixed=TRUE),
                y$ID
            ))
    }
   
    res[vapply(res, length, numeric(1)) != 0]
}


##' @method [ enrichResult
##' @export
`[.enrichResult` <- function(x, i, j, asis = FALSE, ...) {
    x <- get_enriched(x)
    y <- x@result[i, j, ...]
    if (!asis)
        return(y)
    x@result <- y
    return(x)
}

##' @method [ gseaResult
##' @export
`[.gseaResult` <- function(x, i, j, asis = FALSE, ...) {
    y <- x@result[i, j, ...]
    if (!asis)
        return(y)
    x@result <- y
    return(x)
}


##' @method $ enrichResult
##' @export
`$.enrichResult` <-  function(x, name) {
    x <- get_enriched(x)
    x@result[, name]
}

##' @method $ gseaResult
##' @export
`$.gseaResult` <- function(x, name) {
    x@result[, name]
}



##' @method [[ enrichResult
##' @export
`[[.enrichResult` <- function(x, i) {
    gc <- geneInCategory(x)
    if (!i %in% names(gc))
        stop("input term not found...")
    gc[[i]]
}


##' @method [[ gseaResult
##' @export
`[[.gseaResult` <- function(x, i) {
    gc <- geneInCategory(x)
    if (!i %in% names(gc))
        stop("input term not found...")
    gc[[i]]
}


##' @importFrom utils head
##' @method head enrichResult
##' @export
head.enrichResult <- function(x, n=6L, ...) {
    x <- get_enriched(x)
    head(x@result, n, ...)
}

##' @method head gseaResult
##' @export
head.gseaResult <- function(x, n=6L, ...) {
    head(x@result, n, ...)
}

##' @importFrom utils tail
##' @method tail enrichResult
##' @export
tail.enrichResult <- function(x, n=6L, ...) {
    x <- get_enriched(x)
    tail(x@result, n, ...)
}

##' @method tail gseaResult
##' @export
tail.gseaResult <- function(x, n=6L, ...) {
    tail(x@result, n, ...)
}

##' @method dim enrichResult
##' @export
dim.enrichResult <- function(x) {
    x <- get_enriched(x)
    dim(x@result)
}

##' @method dim gseaResult
##' @export
dim.gseaResult <- function(x) {
    dim(x@result)
}




##' summary method for \code{gseaResult} instance
##'
##'
##' @name summary
##' @docType methods
##' @rdname summary-methods
##'
##' @title summary method
##' @return A data frame
##' @exportMethod summary
##' @usage summary(object, ...)
##' @author Guangchuang Yu \url{https://guangchuangyu.github.io}
setMethod("summary", signature(object="gseaResult"),
          function(object, ...) {
              warning("summary method to convert the object to data.frame is deprecated, please use as.data.frame instead.")
              return(as.data.frame(object, ...))
          }
          )


##' summary method for \code{enrichResult} instance
##'
##'
##' @name summary
##' @docType methods
##' @rdname summary-methods
##'
##' @title summary method
##' @param object A \code{enrichResult} instance.
##' @param ... additional parameter
##' @return A data frame
##' @exportMethod summary
##' @usage summary(object, ...)
##' @author Guangchuang Yu \url{http://guangchuangyu.github.io}
setMethod("summary", signature(object="enrichResult"),
          function(object, ...) {
              warning("summary method to convert the object to data.frame is deprecated, please use as.data.frame instead.")
              return(as.data.frame(object, ...))
          }
          )

##' @importFrom fgsea fgsea
GSEA_fgsea <- function(geneList,
                       exponent,
                       nPerm,
                       minGSSize,
                       maxGSSize,
                       eps,
                       pvalueCutoff,
                       pAdjustMethod,
                       verbose,
                       seed=FALSE,
                       USER_DATA,
                       ...) {

    if(verbose) {
        message("using 'fgsea' for GSEA analysis, please cite Korotkevich et al (2019).\n")
        message("preparing geneSet collections...")
    }

    geneSets <- getGeneSet(USER_DATA)
    if (!check_gene_id(geneList, geneSets)) return(NULL)

    if(verbose)
        message("GSEA analysis...")

    if (seed)
        set.seed(.Random.seed)
    
    if(missing(nPerm)){
        tmp_res <- fgsea(pathways=geneSets,
                         stats=geneList,
                         minSize=minGSSize,
                         maxSize=maxGSSize,
                         eps=eps,
                         gseaParam=exponent,
                         nproc = 0,
                         ...)
    } else {
        warning("We do not recommend using nPerm parameter in",
                "current and future releases")
        tmp_res <- fgsea(pathways=geneSets,
                         stats=geneList,
                         nperm=nPerm,
                         minSize=minGSSize,
                         maxSize=maxGSSize,
                         gseaParam=exponent,
                         nproc = 0,
                         ...)

    }

    p.adj <- p.adjust(tmp_res$pval, method=pAdjustMethod)
    qvalues <- calculate_qvalue(tmp_res$pval)

    Description <- TERM2NAME(tmp_res$pathway, USER_DATA)
    
    if(missing(nPerm)){
        params <- list(pvalueCutoff = pvalueCutoff,
                       eps = eps,
                       pAdjustMethod = pAdjustMethod,
                       exponent = exponent,
                       minGSSize = minGSSize,
                       maxGSSize = maxGSSize
                   )
    } else {
        params <- list(pvalueCutoff = pvalueCutoff,
                       nPerm = nPerm,
                       pAdjustMethod = pAdjustMethod,
                       exponent = exponent,
                       minGSSize = minGSSize,
                       maxGSSize = maxGSSize
                   )
    }
    

    res <- data.frame(
        ID = as.character(tmp_res$pathway),
        Description = unname(Description),
        setSize = tmp_res$size,
        enrichmentScore = tmp_res$ES,
        NES = tmp_res$NES,
        pvalue = tmp_res$pval,
        p.adjust = p.adj,
        qvalue = qvalues,
        stringsAsFactors = FALSE
    )

    res <- res[!is.na(res$pvalue),]
    res <- res[ res$pvalue <= pvalueCutoff, ]
    res <- res[ res$p.adjust <= pvalueCutoff, ]
    idx <- order(res$p.adjust, -abs(res$NES), decreasing = FALSE)
    res <- res[idx, ]

    if (nrow(res) == 0) {
        message("no term enriched under specific pvalueCutoff...")
        return(
            new("gseaResult",
                result     = res,
                geneSets   = geneSets,
                geneList   = geneList,
                params     = params,
                readable   = FALSE
                )
        )
    }

    row.names(res) <- res$ID
    observed_info <- lapply(geneSets[res$ID], function(gs)
        gseaScores(geneSet=gs,
                   geneList=geneList,
                   exponent=exponent)
        )

    if (verbose)
        message("leading edge analysis...")

    ledge <- leading_edge(observed_info)

    res$rank <- ledge$rank
    res$leading_edge <- ledge$leading_edge
    res$core_enrichment <- sapply(ledge$core_enrichment, paste0, collapse='/')

    if (verbose)
        message("done...")

    new("gseaResult",
        result     = res,
        geneSets   = geneSets,
        geneList   = geneList,
        params     = params,
        readable   = FALSE
        )
}

##' generic function for gene set enrichment analysis
##'
##'
##' @title GSEA_internal
##' @param geneList order ranked geneList
##' @param exponent weight of each step
##' @param minGSSize minimal size of each geneSet for analyzing
##' @param maxGSSize maximal size of each geneSet for analyzing
##' @param eps This parameter sets the boundary for calculating the p value.
##' @param pvalueCutoff p value Cutoff
##' @param pAdjustMethod p value adjustment method
##' @param verbose print message or not
##' @param seed set seed inside the function to make result reproducible. FALSE by default.
##' @param USER_DATA annotation data
##' @param by one of 'fgsea' or 'DOSE'
##' @param ... other parameter
##' @return gseaResult object
##' @author Yu Guangchuang
GSEA_internal <- function(geneList,
                 exponent,
                 minGSSize,
                 maxGSSize,
                 eps,
                 pvalueCutoff,
                 pAdjustMethod,
                 verbose,
                 seed=FALSE,
                 USER_DATA,
                 by="fgsea",
                 ...) {

    by <- match.arg(by, c("fgsea", "DOSE"))
    if (!is.sorted(geneList))
        stop("geneList should be a decreasing sorted vector...")
    if (by == 'fgsea') {
        .GSEA <- GSEA_fgsea
    } else {
        .GSEA <- GSEA_DOSE
    }
   
    res <- .GSEA(geneList          = geneList,
                 exponent          = exponent,
                 minGSSize         = minGSSize,
                 maxGSSize         = maxGSSize,
                 eps               = eps,
                 pvalueCutoff      = pvalueCutoff,
                 pAdjustMethod     = pAdjustMethod,
                 verbose           = verbose,
                 seed              = seed,
                 USER_DATA         = USER_DATA,
                 ...)

    res@organism <- "UNKNOWN"
    res@setType <- "UNKNOWN"
    res@keytype <- "UNKNOWN"
    if (inherits(USER_DATA, "GSON")) {
        if (!is.null(USER_DATA@keytype)) {
            res@keytype <- USER_DATA@keytype
        }
        if (!is.null(USER_DATA@species)) {
            res@organism <- USER_DATA@species
        } 
        if (!is.null(USER_DATA@gsname)) {
            res@setType <- gsub(".*;", "", USER_DATA@gsname)
        }
    }
    return(res)
}

##' @importFrom utils setTxtProgressBar
##' @importFrom utils txtProgressBar
##' @importFrom stats p.adjust
##' @importFrom BiocParallel bplapply
##' @importFrom BiocParallel MulticoreParam
## @importFrom BiocParallel bpisup
## @importFrom BiocParallel bpstart
## @importFrom BiocParallel bpstop
##' @importFrom BiocParallel multicoreWorkers
GSEA_DOSE <- function(geneList,
                 exponent,
                 nPerm,
                 minGSSize,
                 maxGSSize,
                 pvalueCutoff,
                 pAdjustMethod,
                 verbose,
                 seed=FALSE,
                 USER_DATA,
                 ...) {

    if(verbose)
        message("preparing geneSet collections...")
    geneSets <- getGeneSet(USER_DATA)
    if (!check_gene_id(geneList, geneSets)) return(NULL)


    selected.gs <- geneSet_filter(geneSets, geneList, minGSSize, maxGSSize)

    if (is.null(selected.gs))
        return(NULL)


    if (verbose)
        message("calculating observed enrichment scores...")

    observed_info <- lapply(selected.gs, function(gs)
        gseaScores(geneSet=gs,
                   geneList=geneList,
                   exponent=exponent)
        )
    observedScore <- sapply(observed_info, function(x) x$ES)

    if (verbose) {
        message("calculating permutation scores...")
    }
    if (seed) {
        seeds <- sample.int(nPerm)
    }

    ## if (!bpisup()) {
    ##     bpstart(MulticoreParam(multicoreWorkers(), progressbar=verbose))
    ##     on.exit(bpstop())
    ## }

    permScores <- bplapply(1:nPerm, function(i) {
        if (seed)
            set.seed(seeds[i])
        perm.gseaEScore(geneList, selected.gs, exponent)
    })

    permScores <- do.call("cbind", permScores)

    rownames(permScores) <- names(selected.gs)

    pos.m <- apply(permScores, 1, function(x) mean(x[x >= 0]))
    neg.m <- apply(permScores, 1, function(x) abs(mean(x[x < 0])))


    normalized_ES <- function(ES, pos.m, neg.m) {
        s <- sign(ES)
        m <- numeric(length(ES))
        m[s==1] <- pos.m[s==1]
        m[s==-1] <- neg.m[s==-1]
        ES/m
    }

    NES <- normalized_ES(observedScore, pos.m, neg.m)

    permScores <- apply(permScores, 2, normalized_ES, pos.m=pos.m, neg.m=neg.m)

    if (verbose)
        message("calculating p values...")
    pvals <- sapply(seq_along(observedScore), function(i) {
        if( is.na(NES[i]) ) {
            NA
        } else if ( NES[i] >= 0 ) {
            (sum(permScores[i, ] >= NES[i]) +1) / (sum(permScores[i,] >= 0) +1)
        } else { # NES[i] < 0
            (sum(permScores[i, ] <= NES[i]) +1) / (sum(permScores[i,] < 0) +1)
        }

    })
    p.adj <- p.adjust(pvals, method=pAdjustMethod)
    qvalues <- calculate_qvalue(pvals)

    gs.name <- names(selected.gs)
    Description <- TERM2NAME(gs.name, USER_DATA)

    params <- list(pvalueCutoff = pvalueCutoff,
                   nPerm = nPerm,
                   pAdjustMethod = pAdjustMethod,
                   exponent = exponent,
                   minGSSize = minGSSize,
                   maxGSSize = maxGSSize
                   )


    res <- data.frame(
        ID = as.character(gs.name),
        Description = Description,
        setSize = sapply(selected.gs, length),
        enrichmentScore = observedScore,
        NES = NES,
        pvalue = pvals,
        p.adjust = p.adj,
        qvalue = qvalues,
        stringsAsFactors = FALSE
    )

    res <- res[!is.na(res$pvalue),]
    res <- res[ res$pvalue <= pvalueCutoff, ]
    res <- res[ res$p.adjust <= pvalueCutoff, ]
    idx <- order(res$p.adjust, -abs(res$NES), decreasing = FALSE)
    res <- res[idx, ]

    if (nrow(res) == 0) {
        message("no term enriched under specific pvalueCutoff...")
        return(
            new("gseaResult",
                result     = res,
                geneSets   = geneSets,
                geneList   = geneList,
                params     = params,
                readable   = FALSE
                )
        )
    }

    row.names(res) <- res$ID
    observed_info <- observed_info[res$ID]

    if (verbose)
        message("leading edge analysis...")

    ledge <- leading_edge(observed_info)

    res$rank <- ledge$rank
    res$leading_edge <- ledge$leading_edge
    res$core_enrichment <- sapply(ledge$core_enrichment, paste0, collapse='/')


    if (verbose)
        message("done...")

    new("gseaResult",
        result     = res,
        geneSets   = geneSets,
        geneList   = geneList,
        permScores = permScores,
        params     = params,
        readable   = FALSE
        )

}


leading_edge <- function(observed_info) {
    core_enrichment <- lapply(observed_info, function(x) {
        runningES <- x$runningES
        runningES <- runningES[runningES$position == 1,]
        ES <- x$ES
        if (ES >= 0) {
            i <- which.max(runningES$runningScore)
            leading_gene <- runningES$gene[1:i]
        } else {
            i <- which.min(runningES$runningScore)
            leading_gene <- runningES$gene[-c(1:(i-1))]
        }
        return(leading_gene)
    })

    rank <- sapply(observed_info, function(x) {
        runningES <- x$runningES
        ES <- x$ES
        if (ES >= 0) {
            rr <- which.max(runningES$runningScore)
        } else {
            i <- which.min(runningES$runningScore)
            rr <- nrow(runningES) - i + 1
        }
        return(rr)
    })

    tags <- sapply(observed_info, function(x) {
        runningES <- x$runningES
        runningES <- runningES[runningES$position == 1,]
        ES <- x$ES
        if (ES >= 0) {
            i <- which.max(runningES$runningScore)
            res <- i/nrow(runningES)
        } else {
            i <- which.min(runningES$runningScore)
            res <- (nrow(runningES) - i + 1)/nrow(runningES)
        }
        return(res)
    })

    ll <- sapply(observed_info, function(x) {
        runningES <- x$runningES
        ES <- x$ES
        if (ES >= 0) {
            i <- which.max(runningES$runningScore)
            res <- i/nrow(runningES)
        } else {
            i <- which.min(runningES$runningScore)
            res <- (nrow(runningES) - i + 1)/nrow(runningES)
        }
        return(res)
    })

    N <- nrow(observed_info[[1]]$runningES)
    setSize <- sapply(observed_info, function(x) sum(x$runningES$position))
    signal <- tags * (1-ll) * (N / (N - setSize))

    tags <- paste0(round(tags * 100), "%")
    ll <- paste0(round(ll * 100), "%")
    signal <- paste0(round(signal * 100), "%")
    leading_edge <- paste0('tags=', tags, ", list=", ll, ", signal=", signal)

    res <- list(rank = rank,
                tags = tags,
                list = ll,
                signal = signal,
                leading_edge = leading_edge,
                core_enrichment = core_enrichment)
    return(res)
}

## GSEA algorithm (Subramanian et al. PNAS 2005)
## INPUTs to GSEA
## 1. Expression data set D with N genes and k samples.
## 2. Ranking procedure to produce Gene List L.
## Includes a correlation (or other ranking metric)
## and a phenotype or profile of interest C.
## 3. An exponent p to control the weight of the step.
## 4. Independently derived Gene Set S of N_H genes (e.g., a pathway).
## Enrichment Score ES.
## 2. Evaluate the fraction of genes in S ("hits") weighted
## by their correlation and the fraction of genes not in S ("miss")
## present up to a given position i in L.
gseaScores <- function(geneList, geneSet, exponent=1, fortify=FALSE) {
    ###################################################################
    ##    geneList                                                   ##
    ##                                                               ##
    ## 1. Rank order the N genes in D to form L = { g_1, ... , g_N}  ##
    ##    according to the correlation, r(g_j)=r_j,                  ##
    ##    of their expression profiles with C.                       ##
    ##                                                               ##
    ###################################################################

    ###################################################################
    ##    exponent                                                   ##
    ##                                                               ##
    ## An exponent p to control the weight of the step.              ##
    ##   When p = 0, Enrichment Score ( ES(S) ) reduces to           ##
    ##   the standard Kolmogorov-Smirnov statistic.                  ##
    ##   When p = 1, we are weighting the genes in S                 ##
    ##   by their correlation with C normalized                      ##
    ##   by the sum of the correlations over all of the genes in S.  ##
    ##                                                               ##
    ###################################################################

    ## genes defined in geneSet should appear in geneList.
    ## this is a must, see https://github.com/GuangchuangYu/DOSE/issues/23
    geneSet <- intersect(geneSet, names(geneList))

    N <- length(geneList)
    Nh <- length(geneSet)

    Phit <- Pmiss <- numeric(N)
    hits <- names(geneList) %in% geneSet ## logical

    Phit[hits] <- abs(geneList[hits])^exponent
    NR <- sum(Phit)
    Phit <- cumsum(Phit/NR)

    Pmiss[!hits] <-  1/(N-Nh)
    Pmiss <- cumsum(Pmiss)

    runningES <- Phit - Pmiss

    ## ES is the maximum deviation from zero of Phit-Pmiss
    max.ES <- max(runningES)
    min.ES <- min(runningES)
    if( abs(max.ES) > abs(min.ES) ) {
        ES <- max.ES
    } else {
        ES <- min.ES
    }

    df <- data.frame(x=seq_along(runningES),
                     runningScore=runningES,
                     position=as.integer(hits)
                     )

    if(fortify==TRUE) {
        return(df)
    }

    df$gene = names(geneList)
    res <- list(ES=ES, runningES = df)
    return(res)
}

perm.geneList <- function(geneList) {
    ## perm.idx <- sample(seq_along(geneList), length(geneList), replace=FALSE)
    perm.idx <- sample.int(length(geneList))
    perm.geneList <- geneList
    names(perm.geneList) <- names(geneList)[perm.idx]
    return(perm.geneList)
}

perm.gseaEScore <- function(geneList, geneSets, exponent=1) {
    geneList <- perm.geneList(geneList)
    res <- sapply(1:length(geneSets), function(i)
                  gseaScores(geneSet=geneSets[[i]],
                             geneList=geneList,
                             exponent=exponent)$ES
                  )
    return(res)
}


geneSet_filter <- function(geneSets, geneList, minGSSize, maxGSSize) {
    geneSets <- sapply(geneSets, intersect, names(geneList))

    gs.idx <- get_geneSet_index(geneSets, minGSSize, maxGSSize)
    nGeneSet <- sum(gs.idx)

    if ( nGeneSet == 0 ) {
        msg <- paste0("No gene set have size between [", minGSSize, ", ", maxGSSize, "]...")
        message(msg)
        message("--> return NULL...")
        return(NULL)
    }
    geneSets[gs.idx]
}

##' Enrichment analysis based on the DisGeNET (\url{http://www.disgenet.org/})
##'
##' given a vector of genes, this function will return the enrichment NCG
##' categories with FDR control
##'
##'
##' @title enrichDGN
##' @param snp a vector of SNP
##' @param pvalueCutoff pvalue cutoff
##' @param pAdjustMethod one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"
##' @param universe background genes
##' @param minGSSize minimal size of genes annotated by NCG category for testing
##' @param maxGSSize maximal size of each geneSet for analyzing
##' @param qvalueCutoff qvalue cutoff
##' @param readable whether mapping gene ID to gene Name
##' @return A \code{enrichResult} instance
##' @export
##' @references Janet et al. (2015) DisGeNET: a discovery platform for the dynamical exploration of human diseases and their genes. \emph{Database} bav028
##' \url{http://database.oxfordjournals.org/content/2015/bav028.long}
##' @author Guangchuang Yu
enrichDGNv <- function(snp,
                      pvalueCutoff = 0.05,
                      pAdjustMethod = "BH",
                      universe,
                      minGSSize = 10,
                      maxGSSize = 500,
                      qvalueCutoff = 0.2,
                      readable = FALSE){
    enrichDisease(gene = snp,
                  pvalueCutoff = pvalueCutoff,
                  pAdjustMethod = pAdjustMethod,
                  universe = universe,
                  minGSSize = minGSSize,
                  maxGSSize = maxGSSize,
                  qvalueCutoff = qvalueCutoff,
                  readable = readable,
                  ontology = "snpDisGeNET")
 
}

get_VDGN_data <- function() {
    if (!exists(".DOSEenv")) .initial()
    .DOSEEnv <- get(".DOSEEnv", envir = .GlobalEnv)
    
    if (!exists(".VDGN_DOSE_Env", envir=.DOSEEnv)) {
        tryCatch(utils::data(list="VDGN_EXTID2PATHID", package="DOSE"))
        tryCatch(utils::data(list="VDGN_PATHID2EXTID", package="DOSE"))
        tryCatch(utils::data(list="VDGN_PATHID2NAME", package="DOSE"))
        EXTID2PATHID <- VDGN_EXTID2PATHID <- get("VDGN_EXTID2PATHID")
        PATHID2EXTID <- VDGN_PATHID2EXTID <- get("VDGN_PATHID2EXTID")
        PATHID2NAME <- VDGN_PATHID2NAME <- get("VDGN_PATHID2NAME")

        rm(VDGN_EXTID2PATHID, envir = .GlobalEnv)
        rm(VDGN_PATHID2EXTID, envir = .GlobalEnv)
        rm(VDGN_PATHID2NAME, envir = .GlobalEnv)

        assign(".VDGN_DOSE_Env", new.env(), envir = .DOSEEnv)
        .VDGN_DOSE_Env <- get(".VDGN_DOSE_Env", envir = .DOSEEnv)
        assign("EXTID2PATHID", EXTID2PATHID, envir = .VDGN_DOSE_Env)
        assign("PATHID2EXTID", PATHID2EXTID, envir = .VDGN_DOSE_Env)
        assign("PATHID2NAME", PATHID2NAME, envir = .VDGN_DOSE_Env)
    }
    get(".VDGN_DOSE_Env", envir = .DOSEEnv)
}

#' @keywords internal
"_PACKAGE"


#' Datasets
#'
#' Information content and DO term to entrez gene IDs mapping
#'
#'
#' @name DataSet
#' @aliases  geneList
#' NCG_EXTID2PATHID NCG_PATHID2EXTID NCG_PATHID2NAME
#' DGN_EXTID2PATHID DGN_PATHID2EXTID DGN_PATHID2NAME
#' VDGN_EXTID2PATHID VDGN_PATHID2EXTID VDGN_PATHID2NAME
#' @docType data
#' @keywords datasets
NULL

##' measuring similarities bewteen two gene vectors.
##'
##' provide two entrez gene vectors, this function will calculate their similarity.
##' @title geneSim
##' @param geneID1 entrez gene vector
##' @param geneID2 entrez gene vector
##' @param organism one of "hsa" and "mmu"
##' @param ont one of "HDO" and "MPO"
##' @param measure one of "Wang", "Resnik", "Rel", "Jiang", and "Lin".
##' @param combine One of "max", "avg", "rcmax", "BMA" methods, for combining semantic similarity scores of multiple DO terms associated with gene/protein.
##' @return score matrix
##' @importFrom GOSemSim combineScores
##' @export
##' @examples
##' g <- c("835", "5261","241", "994")
##' geneSim(g)
##' @author Guangchuang Yu \url{https://yulab-smu.top}
geneSim <- function(geneID1,
                    geneID2=NULL,
                    ont = "HDO",
                    organism = "hsa",
                    measure="Wang",
                    combine="BMA") {

    if (ont == "DO") ont <- 'HDO'

    DOID1 <- lapply(geneID1, gene2DO, organism = organism, ont = ont)
    if (is.null(geneID2)) {
        geneID2 <- geneID1
        DOID2 <- DOID1
    } else {
        DOID2 <- lapply(geneID2, gene2DO, organism = organism, ont = ont)
    }

    m <- length(geneID1)
    n <- length(geneID2)
    scores <- matrix(NA, nrow=m, ncol=n)
    rownames(scores) <- geneID1
    colnames(scores) <- geneID2

    for (i in 1:m) {
        if (length(geneID1) == length(geneID2) && all(geneID1 == geneID2)) {
           nn <- i
           flag <- TRUE
        } else {
            flag <- FALSE
            nn <- n
        }

        for (j in 1:nn) {
            if(any(!is.na(DOID1[[i]])) &&  any(!is.na(DOID2[[j]]))) {
                s <- doseSim(DOID1[[i]],
                           DOID2[[j]],
                           measure = measure,
                           ont = ont
                           )
                scores[i,j] = combineScores(s, combine)
                if (flag == TRUE && j != i) {
                    scores[j, i] <- scores[i,j]
                }
            }
        }
    }
    if (nrow(scores) == 1 & ncol(scores) == 1)
        scores = as.numeric(scores)
    return(scores)
}

library(DOSE)

context("geneSim")

test_that("geneSim", {
    res <-  sapply(c("Wang", "Lin", "Jiang", "Resnik", "Rel"), function(mm) geneSim('2524', '3070', measure=mm))
    expect_true(all(res >=0) && all(res <=1))
})

##' show method for \code{gseaResult} instance
##'
##' @name show
##' @docType methods
##' @rdname show-methods
##'
##' @title show method
##' @return message
##' @importFrom methods show
##' @exportMethod show
##' @usage show(object)
##' @author Guangchuang Yu \url{https://yulab-smu.top}
setMethod("show", signature(object="gseaResult"),
          function (object){
              params <- object@params
              cat("#\n# Gene Set Enrichment Analysis\n#\n")
              cat("#...@organism", "\t", object@organism, "\n")
              cat("#...@setType", "\t", object@setType, "\n")
              kt <- object@keytype
              if (kt != "UNKNOWN") {
                  cat("#...@keytype", "\t", kt, "\n")
              }

              cat("#...@geneList", "\t")
              str(object@geneList)
              cat("#...nPerm", "\t", params$nPerm, "\n")
              cat("#...pvalues adjusted by", paste0("'", params$pAdjustMethod, "'"),
                  paste0("with cutoff <", params$pvalueCutoff), "\n")
              cat(paste0("#...", nrow(object@result)), "enriched terms found\n")
              str(object@result)
              cat("#...Citation\n")
              print_citation_msg(object@setType)
          }
)


##' show method for \code{enrichResult} instance
##'
##' @name show
##' @docType methods
##' @rdname show-methods
##'
##' @title show method
##' @param object A \code{enrichResult} instance.
##' @return message
##' @importFrom utils str
##' @importFrom methods show
##' @exportMethod show
##' @usage show(object)
##' @author Guangchuang Yu \url{https://yulab-smu.top}
setMethod("show", signature(object="enrichResult"),
        function (object){
              
              cat("#\n# over-representation test\n#\n")
              cat("#...@organism", "\t", object@organism, "\n")
              cat("#...@ontology", "\t", object@ontology, "\n")
              kt <- object@keytype
              if (kt != "UNKNOWN") {
                  cat("#...@keytype", "\t", kt, "\n")
              }
              cat("#...@gene", "\t")
              str(object@gene)
              cat("#...pvalues adjusted by", paste0("'", object@pAdjustMethod, "'"),
                  paste0("with cutoff <", object@pvalueCutoff), "\n")
              object <- get_enriched(object)
              n <- nrow(object@result)
              cat(paste0("#...", n), "enriched terms found\n")
              if (n > 0) str(object@result)
              cat("#...Citation\n")
              print_citation_msg(object@ontology)
        }
)


print_citation_msg <- function(ontology) {
    refs <- yulab.utils:::ref_knownledge()

    if (ontology == "HDO" || ontology == "NCG") {
        citation_msg <- refs["DOSE"]
    } else if (ontology == "Reactome") {
        citation_msg <- refs["ReactomePA"]
    } else if (ontology == "MeSH") {
        citation_msg <- refs["meshes"]
    } else {
        citation_msg <- refs["clusterProfiler_NP"]
    }
    cat(citation_msg, "\n\n")
}

enrichDisease <- function(gene,
                          organism = "hsa",
                          pvalueCutoff = 0.05,
                          pAdjustMethod = "BH",
                          universe,
                          minGSSize = 10,
                          maxGSSize = 500,
                          qvalueCutoff = 0.2,
                          readable = FALSE,
                          ontology){

    organism <- match.arg(organism, c("hsa", "mm"))

    annoData <- get_anno_data(ontology)
    
    res <- enricher_internal(gene = gene,
                             pvalueCutoff = pvalueCutoff,
                             pAdjustMethod = pAdjustMethod,
                             universe = universe,
                             minGSSize = minGSSize,
                             maxGSSize = maxGSSize,
                             qvalueCutoff = qvalueCutoff,
                             USER_DATA = annoData)

    if (is.null(res))
        return(res)
    if (organism == "hsa") {
        res@organism <- "Homo sapiens"
    } else {
        res@organism <- "Mus musculus"
    }
    
    res@keytype <- "ENTREZID"
    res@ontology <- ontology

    if(readable) {
        if (organism == "hsa") {
            res <- setReadable(res, 'org.Hs.eg.db')
        } else {
            res <- setReadable(res, 'org.Mm.eg.db')
        }
    }
    return(res)
}


get_anno_data <- function(ontology) {
    if (ontology == "NCG") {
        annoData <- get_NCG_data()
    } else if (ontology == "DisGeNET") {
        annoData <- get_DGN_data()
    } else if (ontology == "snpDisGeNET") {
        annoData <- get_VDGN_data()
    } else if (ontology %in% c("HDO", "MPO", "HPO")) {
        annoData <- get_dose_data(ontology)
    } else {
        stop("ontology not supported yet...")
    }
    
    return(annoData)
}

get_dose_data <- function(ontology = "HPO") {
    .DOSEEnv <- get_dose_env()
    .env <- sprintf(".%s_DOSE_Env", ontology)
    if (exists(.env, envir=.DOSEEnv)) {
        res <- get(.env, envir = .DOSEEnv)
        return(res)
    }

    assign(.env, new.env(), envir = .DOSEEnv)
    ret_env <- get(.env, envir = .DOSEEnv)

    TERM2ALLEG <- get_ont2allgene(ontology) 
    EG2ALLTERM <- get_gene2allont(ontology) 

    termmap <- GOSemSim:::get_onto_data(
        ontology, 
        table="term", 
        output = "data.frame")

    PATH2NAME.df <- unique(termmap)
    PATH2NAME <- setNames(PATH2NAME.df[,2], PATH2NAME.df[,1])        

    assign("EXTID2PATHID", EG2ALLTERM, envir = ret_env)
    assign("PATHID2EXTID", TERM2ALLEG, envir = ret_env)
    assign("PATHID2NAME", PATH2NAME, envir = ret_env)

    return(ret_env)    
}

##' parse character ratio to double value, such as 1/5 to 0.2
##'
##' 
##' @title parse_ratio
##' @param ratio character vector of ratio to parse
##' @return A numeric vector (double) of parsed ratio
##' @export
##' @author Guangchuang Yu
parse_ratio <- function(ratio) {
    ratio <- sub("^\\s*", "", as.character(ratio))
    ratio <- sub("\\s*$", "", ratio)
    numerator <- as.numeric(sub("/\\d+$", "", ratio))
    denominator <- as.numeric(sub("^\\d+/", "", ratio))
    return(numerator/denominator)
}

##' semantic similarity between two gene clusters
##'
##' given two gene clusters, this function calculates semantic similarity between them.
##'
##' @title clusterSim
##' @param cluster1 a vector of gene IDs
##' @param cluster2 another vector of gene IDs
##' @param organism one of "hsa" and "mmu"
##' @param ont one of "HDO", "HPO" and "MPO"
##' @param measure One of "Resnik", "Lin", "Rel", "Jiang" and "Wang" methods.
##' @param combine One of "max", "avg", "rcmax", "BMA" methods, for combining
##' @return similarity
##' @importFrom GOSemSim combineScores
##' @export
##' @author Yu Guangchuang
##' @examples
##' \dontrun{
##'	cluster1 <- c("835", "5261","241", "994")
##'	cluster2 <- c("307", "308", "317", "321", "506", "540", "378", "388", "396")
##'	clusterSim(cluster1, cluster2)
##' }
clusterSim <- function(cluster1, 
                       cluster2, 
                       ont = "HDO",
                       organism = "hsa",
                       measure="Wang", 
                       combine="BMA") {
    if (ont == "DO") ont <- 'HDO'

    do1 <- sapply(cluster1, gene2DO, organism = organism)
    do2 <- sapply(cluster2, gene2DO, organism = organism)

    do1 <- unlist(do1)
    do2 <- unlist(do2)

    res <- doseSim(DOID1 = do1, DOID2 = do2, measure = measure, ont = ont)
    combineScores(res, combine)
}

##' mapping geneID to gene Symbol
##'
##'
##' @title setReadable
##' @param x enrichResult Object
##' @param OrgDb OrgDb
##' @param keyType keyType of gene
##' @return enrichResult Object
##' @author Yu Guangchuang
##' @export
setReadable <- function(x, OrgDb, keyType="auto") {
    OrgDb <- load_OrgDb(OrgDb)
    if (!'SYMBOL' %in% columns(OrgDb)) {
        warning("Fail to convert input geneID to SYMBOL since no SYMBOL information available in the provided OrgDb...")
    }

    if (!(is(x, "enrichResult") || is(x, "groupGOResult") || is(x, "gseaResult") || is(x,"compareClusterResult")))
        stop("input should be an 'enrichResult' , 'gseaResult' or 'compareClusterResult' object...")

    isGSEA <- FALSE
    isCompare <- FALSE
    if (is(x, 'gseaResult'))
        isGSEA <- TRUE

    if (is(x, 'compareClusterResult'))
        isCompare <- TRUE

    if (keyType == "auto") {
        keyType <- x@keytype
        if (keyType == 'UNKNOWN') {
            stop("can't determine keyType automatically; need to set 'keyType' explicitly...")
        }
    }

    if (x@readable)
        return(x)

    gc <- geneInCategory(x)
    if (isGSEA) {
        genes <- names(x@geneList)
    } else if (isCompare) {
        if ("core_enrichment" %in% colnames(as.data.frame(x))) {
            geneslist <- x@geneClusters
            names(geneslist) <- NULL
            genes <- unique(names(unlist(geneslist)))
        } else {
            genes <- unique(unlist(x@geneClusters))
        }     
    } else {
        genes <- x@gene
    }

    gn <- EXTID2NAME(OrgDb, genes, keyType)


    if(isCompare) {
        gc2 <- list()
        k <- 1
        for(i in seq_len(length(gc))) {
            for(j in seq_len(length(gc[[i]]))) {
                gc2[[k]] <- gc[[i]][[j]]
                names(gc2)[k] <- paste(names(gc)[[i]], names(gc[[i]])[j], sep="-")
                k <- k + 1
            }
        }
        gc <- gc2
        gc <- lapply(gc, function(i) gn[i])
        res <- x@compareClusterResult
        gc <- gc[paste(res$Cluster, res$ID, sep= "-")]
    } else {
        gc <- lapply(gc, function(i) gn[i])
        res <- x@result
        gc <- gc[as.character(res$ID)]
    }

    ## names(gc) should be identical to res$ID

    ## gc <- gc[as.character(res$ID)]


    geneID <- sapply(gc, paste0, collapse="/")
    # if (isGSEA) {
    if ("core_enrichment" %in% colnames(as.data.frame(x))) {
        res$core_enrichment <- unlist(geneID)
    } else {
        res$geneID <- unlist(geneID)
    }
    x@gene2Symbol <- gn
    x@keytype <- keyType
    x@readable <- TRUE
    if(isCompare){
        x@compareClusterResult <- res
    } else {
        x@result <- res
    }


    return(x)
}


# geneInCategory2 <- function(x){
    # setNames(strsplit(geneID(x), "/", fixed=TRUE),
        # paste(x@compareClusterResult$Cluster,
            # x@compareClusterResult$ID, sep= "-"))
# }

gseDisease <- function(geneList,
                       organism = "hsa",
                       exponent=1,
                       minGSSize = 10,
                       maxGSSize = 500,
                       eps = 1e-10,
                       pvalueCutoff=0.05,
                       pAdjustMethod="BH",
                       verbose=TRUE,
                       seed=FALSE,
                       by = 'fgsea',
                       ontology,
                       ...) {

    annoData <- get_anno_data(ontology)

    res <- GSEA_internal(geneList          = geneList,
                         exponent          = exponent,
                         minGSSize         = minGSSize,
                         maxGSSize         = maxGSSize,
                         eps               = eps,
                         pvalueCutoff      = pvalueCutoff,
                         pAdjustMethod     = pAdjustMethod,
                         verbose           = verbose,
                         seed              = seed,
                         USER_DATA         = annoData,
                         by                = by,
                         ...)

    if (is.null(res))
        return(res)

    if (organism == "hsa") {
        res@organism <- "Homo sapiens"
    } else {
        res@organism <- "Mus musculus"
    }
    res@setType <- ontology
    res@keytype <- "ENTREZID"
    return(res)
}

##' DO Gene Set Enrichment Analysis
##'
##'
##' perform gsea analysis
##' @param geneList order ranked geneList
##' @param ont one of "HDO", "HPO" or "MPO"
##' @param organism one of "hsa" and "mmu"
##' @param exponent weight of each step
##' @param minGSSize minimal size of each geneSet for analyzing
##' @param maxGSSize maximal size of each geneSet for analyzing
##' @param pvalueCutoff pvalue Cutoff
##' @param pAdjustMethod p value adjustment method
##' @param verbose print message or not
##' @param seed logical
##' @param by one of 'fgsea' or 'DOSE'
##' @param ... other parameter
##' @return gseaResult object
##' @export
##' @author Yu Guangchuang
##' @keywords manip
gseDO <- function(geneList,
                  ont = "HDO",
                  organism = "hsa",
                  exponent=1,
                  minGSSize = 10,
                  maxGSSize = 500,
                  pvalueCutoff=0.05,
                  pAdjustMethod="BH",
                  verbose=TRUE,
                  seed=FALSE,
                  by = 'fgsea', 
                  ...) {
     

    gseDisease(geneList          = geneList,
               exponent          = exponent,
               minGSSize         = minGSSize,
               maxGSSize         = maxGSSize,
               pvalueCutoff      = pvalueCutoff,
               pAdjustMethod     = pAdjustMethod,
               verbose           = verbose,
               seed              = seed,
               by                = by,
               ontology          = ont, 
               ...)

}

##' NCG Gene Set Enrichment Analysis
##'
##'
##' perform gsea analysis
##' @inheritParams gseDO
##' @return gseaResult object
##' @export
##' @author Yu Guangchuang
##' @keywords manip
gseNCG <- function(geneList,
                   exponent=1,
                   minGSSize = 10,
                   maxGSSize = 500,
                   pvalueCutoff=0.05,
                   pAdjustMethod="BH",
                   verbose=TRUE,
                   seed=FALSE,
                   by = 'fgsea',
                   ...) {
                  

    gseDisease(geneList          = geneList,
               exponent          = exponent,
               minGSSize         = minGSSize,
               maxGSSize         = maxGSSize,
               pvalueCutoff      = pvalueCutoff,
               pAdjustMethod     = pAdjustMethod,
               verbose           = verbose,
               seed              = seed,
               by                = by,
               ontology          = "NCG", 
               ...)
    


}

##' DisGeNET Gene Set Enrichment Analysis
##'
##'
##' perform gsea analysis
##' @inheritParams gseDO
##' @return gseaResult object
##' @export
##' @author Yu Guangchuang
##' @keywords manip
gseDGN <- function(geneList,
                   exponent=1,
                   minGSSize = 10,
                   maxGSSize = 500,
                   pvalueCutoff=0.05,
                   pAdjustMethod="BH",
                   verbose=TRUE,
                   seed=FALSE,
                   by = 'fgsea',
                   ...) {
                   

    gseDisease(geneList          = geneList,
               exponent          = exponent,
               minGSSize         = minGSSize,
               maxGSSize         = maxGSSize,
               pvalueCutoff      = pvalueCutoff,
               pAdjustMethod     = pAdjustMethod,
               verbose           = verbose,
               seed              = seed,
               by                = by,
               ontology          = "DisGeNET",
               ...)
}

# download from https://www.disgenet.org/downloads
# 1. ALL gene-disease associations
# 2. ALL variant-disease associations
#x <- read.delim("all_gene_disease_associations.tsv", comment.char="#", stringsAsFactor=F)
x <- read.delim("all_gene_disease_associations.tsv", comment.char="#", stringsAsFactor=F, encoding = "latin1")
d2n <- unique(x[, c("diseaseId", "diseaseName")])
d2g <- unique(x[, c("diseaseId", "geneId")])

.DGN_DOSE_Env <- DOSE:::build_Anno(d2g, d2n)


## save(.DGN_DOSE_Env, file="DGN_DOSE_Env.rda", compress='xz')

DGN_EXTID2PATHID = get("EXTID2PATHID", envir=.DGN_DOSE_Env)
DGN_PATHID2EXTID = get("PATHID2EXTID", envir = .DGN_DOSE_Env)
DGN_PATHID2NAME = get("PATHID2NAME", envir = .DGN_DOSE_Env)

## Warning: found non-ASCII strings
## 'Primary Sj<ef><bf><bd>gren's syndrome' in object 'DGN_PATHID2NAME'
## 'Secondary Sj<ef><bf><bd>gren's syndrome' in object 'DGN_PATHID2NAME'
## 'Henoch-Sch<ef><bf><bd>nlein nephritis' in object 'DGN_PATHID2NAME'
##
## Sjögren
##
## DGN_PATHID2NAME['umls:C0151449'] <- "Primary Sjogren's syndrome"
## DGN_PATHID2NAME['umls:C0151450'] <- "Secondary Sjogren's syndrome"
## DGN_PATHID2NAME['umls:C0403528'] <- 'Henoch-Schonlein nephritis'

DGN_PATHID2NAME <- iconv(DGN_PATHID2NAME, "ASCII", "UTF-8") 

save(DGN_EXTID2PATHID, file = "DGN_EXTID2PATHID.rda", compress='xz')
save(DGN_PATHID2EXTID, file="DGN_PATHID2EXTID.rda", compress='xz')
save(DGN_PATHID2NAME, file="DGN_PATHID2NAME.rda", compress='xz')



y <- read.delim("all_variant_disease_associations.tsv", comment.char="#", stringsAsFactor=F)
d2n <- unique(y[, c("diseaseId", "diseaseName")])
d2s <- unique(y[, c("diseaseId", "snpId")])


.VDGN_DOSE_Env <- DOSE:::build_Anno(d2s, d2n)

## save(.VDGN_DOSE_Env, file="VDGN_DOSE_Env.rda", compress='xz')


VDGN_EXTID2PATHID = get("EXTID2PATHID", envir=.VDGN_DOSE_Env)
VDGN_PATHID2EXTID = get("PATHID2EXTID", envir = .VDGN_DOSE_Env)
VDGN_PATHID2NAME = get("PATHID2NAME", envir = .VDGN_DOSE_Env)

VDGN_PATHID2NAME <- iconv(VDGN_PATHID2NAME, "ASCII", "UTF-8")

save(VDGN_EXTID2PATHID, file = "VDGN_EXTID2PATHID.rda", compress='xz')
save(VDGN_PATHID2EXTID, file="VDGN_PATHID2EXTID.rda", compress='xz')
save(VDGN_PATHID2NAME, file="VDGN_PATHID2NAME.rda", compress='xz')

build_Anno <- function(path2gene, path2name) {
    if (!exists(".Anno_clusterProfiler_Env", envir = .GlobalEnv)) {
        pos <- 1
        envir <- as.environment(pos) 
        assign(".Anno_clusterProfiler_Env", new.env(), envir = envir)
    }
    Anno_clusterProfiler_Env <- get(".Anno_clusterProfiler_Env", envir= .GlobalEnv)

    # if(class(path2gene[[2]]) == 'list') {
    if (inherits(path2gene[[2]], "list")){
        ## to compatible with tibble
        path2gene <- cbind(rep(path2gene[[1]],
                               times = vapply(path2gene[[2]], length, numeric(1))),
                           unlist(path2gene[[2]]))
    }

    path2gene <- as.data.frame(path2gene) 
    path2gene <- path2gene[!is.na(path2gene[,1]), ]
    path2gene <- path2gene[!is.na(path2gene[,2]), ]
    path2gene <- unique(path2gene)
    
    PATHID2EXTID <- split(as.character(path2gene[,2]), as.character(path2gene[,1]))
    EXTID2PATHID <- split(as.character(path2gene[,1]), as.character(path2gene[,2]))
    
    assign("PATHID2EXTID", PATHID2EXTID, envir = Anno_clusterProfiler_Env)
    assign("EXTID2PATHID", EXTID2PATHID, envir = Anno_clusterProfiler_Env)

    if ( missing(path2name) || is.null(path2name) || all(is.na(path2name))) {
        assign("PATHID2NAME", NULL, envir = Anno_clusterProfiler_Env)
    } else {
        path2name <- as.data.frame(path2name)
        path2name <- path2name[!is.na(path2name[,1]), ]
        path2name <- path2name[!is.na(path2name[,2]), ]
	path2name <- unique(path2name)
	PATH2NAME <- as.character(path2name[,2])
	names(PATH2NAME) <- as.character(path2name[,1]) 
        assign("PATHID2NAME", PATH2NAME, envir = Anno_clusterProfiler_Env)
    }
    return(Anno_clusterProfiler_Env)
}

##' Class "compareClusterResult"
##' This class represents the comparison result of gene clusters by GO
##' categories at specific level or GO enrichment analysis.
##'
##'
##' @name compareClusterResult-class
##' @aliases compareClusterResult-class show,compareClusterResult-method
##'   summary,compareClusterResult-method plot,compareClusterResult-method
##' @docType class
##' @slot compareClusterResult cluster comparing result
##' @slot geneClusters a list of genes
##' @slot fun one of groupGO, enrichGO and enrichKEGG
##' @slot gene2Symbol gene ID to Symbol
##' @slot keytype Gene ID type
##' @slot readable logical flag of gene ID in symbol or not.
##' @slot .call function call
##' @slot termsim Similarity between term
##' @slot method method of calculating the similarity between nodes
##' @slot dr dimension reduction result
##' @exportClass compareClusterResult
##' @author Guangchuang Yu \url{https://yulab-smu.top}
##' @exportClass compareClusterResult
##' @seealso 
##'   \code{\linkS4class{enrichResult}}
##' @keywords classes
setClass("compareClusterResult",
         representation = representation(
             compareClusterResult = "data.frame",
             geneClusters = "list",
             fun = "character",
             gene2Symbol    = "character",
             keytype        = "character",
             readable       = "logical",
             .call          = "call",
             termsim        = "matrix",
             method         = "character",
             dr             = "list"
         )
         )

##' Class "enrichResult"
##' This class represents the result of enrichment analysis.
##'
##'
##' @name enrichResult-class
##' @aliases enrichResult-class
##'   show,enrichResult-method summary,enrichResult-method
##'
##' @docType class
##' @slot result enrichment analysis
##' @slot pvalueCutoff pvalueCutoff
##' @slot pAdjustMethod pvalue adjust method
##' @slot qvalueCutoff qvalueCutoff
##' @slot organism only "human" supported
##' @slot ontology biological ontology
##' @slot gene Gene IDs
##' @slot keytype Gene ID type
##' @slot universe background gene
##' @slot gene2Symbol mapping gene to Symbol
##' @slot geneSets gene sets
##' @slot readable logical flag of gene ID in symbol or not.
##' @slot termsim Similarity between term
##' @slot method method of calculating the similarity between nodes
##' @slot dr dimension reduction result
##' @exportClass enrichResult
##' @author Guangchuang Yu \url{https://yulab-smu.top}
##' @seealso \code{\link{enrichDO}}
##' @keywords classes
setClass("enrichResult",
         representation=representation(
             result         = "data.frame",
             pvalueCutoff   = "numeric",
             pAdjustMethod  = "character",
             qvalueCutoff   = "numeric",
             organism       = "character",
             ontology       = "character",
             gene           = "character",
             keytype        = "character",
             universe       = "character",
             gene2Symbol    = "character",
             geneSets       = "list",
             readable       = "logical",
             termsim        = "matrix",
             method         = "character",
             dr             = "list"
             ),
         prototype=prototype(readable = FALSE)
         )



##' Class "gseaResult"
##' This class represents the result of GSEA analysis
##'
##'
##' @name gseaResult-class
##' @aliases gseahResult-class
##'   show,gseaResult-method summary,gseaResult-method
##'
##' @docType class
##' @slot result GSEA anaysis
##' @slot organism organism
##' @slot setType setType
##' @slot geneSets geneSets
##' @slot geneList order rank geneList
##' @slot keytype ID type of gene
##' @slot permScores permutation scores
##' @slot params parameters
##' @slot gene2Symbol gene ID to Symbol
##' @slot readable whether convert gene ID to symbol
##' @slot dr dimension reduction result
##' @exportClass gseaResult
##' @author Guangchuang Yu \url{https://yulab-smu.top}
##' @keywords classes
setClass("gseaResult",
         representation   = representation(
             result          = "data.frame",
             organism        = "character",
             setType         = "character",
             geneSets        = "list",
             geneList        = "numeric",
             keytype         = "character",
             permScores      = "matrix",
             params          = "list",
             gene2Symbol     = "character",
             readable        = "logical",
             termsim         = "matrix",
             method          = "character",
             dr              = "list"
         )
         )

get_dose_env <- function() {
    if (!exists(".DOSEEnv")) {
        .initial()
    }
    get(".DOSEEnv")
}

.initial <- function() {
    pos <- 1
    envir <- as.environment(pos)
    assign(".DOSEEnv", new.env(), envir = envir) 
}

# https://github.com/YuLab-SMU/ReactomePA/issues/43
check_gene_id <- function(geneList, geneSets) {
    if (all(!names(geneList) %in% unique(unlist(geneSets)))) {
        sg <- unlist(geneSets[1:10])
        sg <- sample(sg, min(length(sg), 6))
        message("--> Expected input gene ID: ", paste0(sg, collapse=','))
        message("--> No gene can be mapped....")
        return(FALSE)
    }
    return(TRUE)
}


## @importFrom S4Vectors metadata
get_organism <- function(OrgDb) {
    OrgDb <- load_OrgDb(OrgDb)
    ## md <- S4Vectors::metadata(OrgDb)
    ## md[md[,1] == "ORGANISM", 2]
    AnnotationDbi::species(OrgDb)
}


calculate_qvalue <- function(pvals) {
    if (length(pvals) == 0)
        return(numeric(0))

    qobj <- tryCatch(qvalue(pvals, lambda=0.05, pi0.method="bootstrap"), error=function(e) NULL)

    # if (class(qobj) == "qvalue") {
    if (inherits(qobj, "qvalue")) {
        qvalues <- qobj$qvalues
    } else {
        qvalues <- NA
    }
    return(qvalues)
}


calculate_qvalue <- function(pvals) {
    if (length(pvals) == 0)
        return(numeric(0))

    qobj <- tryCatch(qvalue(pvals, lambda=0.05, pi0.method="bootstrap"), error=function(e) NULL)
  
    # if (class(qobj) == "qvalue") {
    if (inherits(qobj, "qvalue")) {
        qvalues <- qobj$qvalues
    } else {
        qvalues <- NA
    }
    return(qvalues)
}

##' compute information content
##'
##'
##' @title compute information content
##' @param ont one of "DO", "HPO" and "MPO"
##' @return NULL
##' @importMethodsFrom AnnotationDbi toTable
##' @author Guangchuang Yu \url{https://yulab-smu.top}
computeIC <- function(ont="HDO"){
    DO2EG <- get_ont2gene(ont)
    Offsprings <- GOSemSim:::getOffsprings(ont)
    
    docount <- unlist(lapply(DO2EG, length))
    doids <- names(docount) 
    
    cnt <- docount[doids] + sapply(doids, function(i) sum(docount[Offsprings[[i]]], na.rm=TRUE))
    names(cnt) <- doids
    p <- cnt/sum(docount)

    ## IC of DO terms was quantified as the negative log likelihood.
    IC <- -log(p)
    return(IC)
}


##' provide gene ID, this function will convert to the corresponding DO Terms
##'
##'
##' @title convert Gene ID to DO Terms
##' @param gene entrez gene ID
##' @param organism organism
##' @param ont ont
##' @return DO Terms
##' @importMethodsFrom AnnotationDbi get
##' @importMethodsFrom AnnotationDbi exists
##' @export
##' @author Guangchuang Yu \url{https://yulab-smu.top}
gene2DO <- function(gene, organism = "hsa", ont = "HDO") {
    gene <- as.character(gene)

    EG2DO <- get_gene2ont(ont)

    DO <- EG2DO[[gene]]
    DO <- unlist(DO)
    if (is.null(DO)) {
        return(NA)
    }
    if (sum(!is.na(DO)) == 0) {
        return(NA)
    }
    DO <- DO[!is.na(DO)]
    if (length(DO) == 0) {
        return(NA)
    }
    return(DO)
}

process_tcss <- getFromNamespace("process_tcss", "GOSemSim")

##' @importClassesFrom GOSemSim GOSemSimDATA
semdata <- function(processTCSS = FALSE, ont = "HDO") {
    IC <- new("GOSemSimDATA",
                ont = ont,
                IC = computeIC(ont = ont))

    if (processTCSS) {
        IC <- IC@IC
        IC@tcssdata <- process_tcss(ont = ont, IC = IC, cutoff = NULL)
    }

    IC
}

semdata2 <- memoise::memoise(semdata)


get_ont2gene <- function(ontology, output = "list") {
    gene2ont <- get_gene2ont(ontology, output = "data.frame")
    if (output == "data.frame") {
        return(gene2ont[, 2:1])
    }

    split(as.character(gene2ont[,1]), as.character(gene2ont[,2]))
}

get_gene2ont <- function(ontology, output = "list") {
    ont2gene <- GOSemSim:::get_onto_data(ontology, table = "ont2gene", output = 'data.frame')
    anc <- GOSemSim:::getAncestors(ontology)
    idx <- ont2gene[,1] %in% names(anc)
    ont2gene <- unique(ont2gene[idx, ])

    if (output == "data.frame") {
        return(ont2gene[, 2:1])
    }

    split(as.character(ont2gene[,1]), as.character(ont2gene[,2]))
}

get_gene2allont <- function(ontology, output = "list") {
    GOSemSim:::get_onto_data(ontology, table = "gene2allont", output = output)
}

get_ont2allgene <- function(ontology, output = "list") {
    gene2allont <- GOSemSim:::get_onto_data(ontology, table = "gene2allont", output = "data.frame")
    if (output == "data.frame") {
        return(gene2allont[, 2:1])
    }

    split(as.character(gene2allont[,1]), as.character(gene2allont[,2]))
}

## ##' get all entrezgene ID of a specific organism
## ##'
## ##'
## ##' @title getALLEG
## ##' @param organism species
## ##' @return entrez gene ID vector
## ##' @export
## ##' @author Yu Guangchuang
## getALLEG <- function(organism) {
##     annoDb <- getDb(organism)
##     require(annoDb, character.only = TRUE)
##     annoDb <- eval(parse(text=annoDb))
##     eg=keys(annoDb, keytype="ENTREZID")
##     return(eg)
## }


##' mapping gene ID to gene Symbol
##'
##'
##' @title EXTID2NAME
##' @param OrgDb OrgDb
##' @param geneID entrez gene ID
##' @param keytype keytype
##' @return gene symbol
##' @importMethodsFrom AnnotationDbi select
##' @importMethodsFrom AnnotationDbi keys
##' @importMethodsFrom AnnotationDbi columns
##' @importMethodsFrom AnnotationDbi keytypes
##' @importFrom GOSemSim load_OrgDb
##' @export
##' @author Guangchuang Yu \url{https://yulab-smu.top}
EXTID2NAME <- function(OrgDb, geneID, keytype) {
    OrgDb <- load_OrgDb(OrgDb)
    kt <- keytypes(OrgDb)
    if (! keytype %in% kt) {
        stop("keytype is not supported...")
    }

    gn.df <- suppressMessages(select(OrgDb, keys=geneID, keytype=keytype, columns="SYMBOL"))
    gn.df <- unique(gn.df)
    colnames(gn.df) <- c("GeneID", "SYMBOL")

    unmap_geneID <- geneID[!geneID %in% gn.df$GeneID]
    if (length(unmap_geneID) != 0) {
        unmap_geneID.df = data.frame(GeneID = unmap_geneID,
                                     SYMBOL = unmap_geneID)
        gn.df <- rbind(gn.df, unmap_geneID.df)
    }

    gn <- gn.df$SYMBOL
    names(gn) <- gn.df$GeneID
    return(gn)
}

## EXTID2NAME <- function(geneID, organism) {
##     if (length(geneID) == 0) {
##         return("")
##     }
##     if (organism == "worm") {
##         organism = "celegans"
##         warning("'worm' is deprecated, please use 'celegans' instead...")
##     }
##     organism <- organismMapper(organism)

##     supported_Org <- getSupported_Org()
##     if (organism %in% supported_Org) {
##         ## kk <- getALLEG(organism)
##         ## unmap_geneID <- geneID[! geneID %in% kk]
##         ## map_geneID <- geneID[geneID %in% kk]

##         ## if (length(map_geneID) == 0) {
##         ##     warning("the input geneID is not entrezgeneID, and cannot be mapped")
##         ##     names(geneID) <- geneID
##         ##     return (geneID)
##         ## }
##         annoDb <- getDb(organism)
##         require(annoDb, character.only = TRUE)
##         annoDb <- eval(parse(text=annoDb))
##         if (organism == "yeast" || organism == "malaria") {
##             gn.df <- select(annoDb, keys=geneID,keytype="ORF", columns="GENENAME")
##         } else if (organism == "arabidopsis") {
##             gn.df <- select(annoDb, keys=geneID,keytype="TAIR", columns="SYMBOL")
##         } else {
##             gn.df <- select(annoDb, keys=geneID,keytype="ENTREZID", columns="SYMBOL")
##         }
##         gn.df <- unique(gn.df)
##         colnames(gn.df) <- c("ENTREZID", "SYMBOL")

##         unmap_geneID <- geneID[!geneID %in% gn.df$ENTREZID]
##         if (length(unmap_geneID) != 0) {
##             unmap_geneID.df = data.frame(ENTREZID= unmap_geneID, SYMBOL=unmap_geneID)
##             gn.df <- rbind(gn.df, unmap_geneID.df)
##         }

##         gn <- gn.df$SYMBOL
##         names(gn) <- gn.df$ENTREZID
##         ##gn <- unique(gn[!is.na(gn)])
##     } else {
##         oldwd <- getwd()
##         if(organism == "D39") {
##             dir <- system.file("extdata/D39/", package="clusterProfiler")
##             setwd(dir)
##         }
##         if(organism == "M5005") {
##             dir <- system.file("extdata/M5005/", package="clusterProfiler")
##             setwd(dir)
##         }

##         if (file.exists("geneTable.rda")) {
##             geneTable <- NULL # to satisfy codetools
##             load("geneTable.rda")
##             idx <- geneTable$GeneID %in% geneID
##             eg.gn <- geneTable[idx, c("GeneID", "GeneName", "Locus")]
##             eg.gn[eg.gn[,2] == "-",2] <- eg.gn[eg.gn[,2] == "-",3]
##             ##eg.gn <- eg.gn[,c(1,2)]
##             gn <- eg.gn$GeneName
##             names(gn) <- as.character(eg.gn$GeneID)
##             setwd(oldwd)
##         } else {
##             setwd(oldwd)
##             warning("Have no annotation found for the input geneID")
##             return(geneID)
##         }
##     }
##     return(gn)
## }



is.sorted <- function(x, decreasing=TRUE) {
    all( sort(x, decreasing=decreasing) == x )
}

getGeneSet <- function(USER_DATA) {
    if (inherits(USER_DATA, "environment")) { 
        res <- get("PATHID2EXTID", envir = USER_DATA)
    } else if (inherits(USER_DATA, "GSON")) {
        gsid2gene <- USER_DATA@gsid2gene
        res <- split(gsid2gene$gene, gsid2gene$gsid) 
    } else {
        stop("not supported")
    }
    return(res)
}


##' @importFrom ggplot2 facet_grid
##' @export
ggplot2::facet_grid

##' plotting similarity matrix
##'
##'
##' @title simplot
##' @param sim similarity matrix
##' @param xlab xlab
##' @param ylab ylab
##' @param color.low color of low value
##' @param color.high color of high value
##' @param labs logical, add text label or not
##' @param digits round digit numbers
##' @param labs.size lable size
##' @param font.size font size
##' @return ggplot object
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 aes
##' @importFrom ggplot2 geom_tile
##' @importFrom ggplot2 geom_text
##' @importFrom ggplot2 scale_fill_gradient
##' @importFrom ggplot2 scale_x_discrete
##' @importFrom ggplot2 scale_y_discrete
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 element_blank
##' @importFrom ggplot2 element_text
##' @importFrom ggplot2 xlab
##' @importFrom ggplot2 ylab
##' @importFrom reshape2 melt
##' @export
##' @author Yu Guangchuang
simplot <- function(sim, xlab="", ylab="", color.low="white", color.high="red", labs=TRUE, digits=2, labs.size=3, font.size=14){
    sim.df <- as.data.frame(sim)
    ## if(readable == TRUE) {
    ##     rownames(sim.df) <- TERM2NAME(rownames(sim.df))
    ##     colnames(sim.df) <- TERM2NAME(colnames(sim.df))
    ## }
    rn <- row.names(sim.df)

    sim.df <- cbind(ID=rownames(sim.df), sim.df)
    sim.df <- melt(sim.df)

    sim.df[,1] <- factor(sim.df[,1], levels=rev(rn))
    if (labs == TRUE) {
        ## lbs <- c(apply(round(sim, digits), 2, as.character))
        sim.df$label <- as.character(round(sim.df$value, digits))
    }
    variable <- ID <- value <- label <- NULL ## to satisfy codetools
    if (labs == TRUE)
        p <- ggplot(sim.df, aes(variable, ID, fill=value, label=label))
    else
        p <- ggplot(sim.df, aes(variable, ID, fill=value))

     p <- p + geom_tile(color="black")+
            scale_fill_gradient(low=color.low, high=color.high) +
                scale_x_discrete(expand=c(0,0)) +
                    scale_y_discrete(expand=c(0,0))+
                        theme(axis.ticks=element_blank())
    if (labs == TRUE)
        p <- p+geom_text(size=labs.size)
    p <- p+theme_dose(font.size)
    p <- p + theme(axis.text.x=element_text(hjust=0, angle=-90)) +
        theme(axis.text.y=element_text(hjust=0))
    p <- p+theme(legend.title=element_blank())
    ##geom_point(aes(size=value))
    p <- p+xlab(xlab)+ylab(ylab)

    ## if (readable == TRUE) {
    ##     p <- p + theme(axis.text.y = element_text(hjust=1))
    ## }
    p <- p + theme(axis.text.x = element_text(vjust=0.5))
    return(p)
}


##' ggplot theme of DOSE
##'
##' @title theme_dose
##' @param font.size font size
##' @return ggplot theme
##' @importFrom ggplot2 theme_bw
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 element_text
##' @importFrom ggplot2 margin
##' @examples
##' library(ggplot2)
##' qplot(1:10) + theme_dose()
##' @export
theme_dose <- function(font.size=14) {
    theme_bw() +
        theme(axis.text.x = element_text(colour = "black",
                                         size = font.size, vjust =1 ),
              axis.text.y = element_text(colour = "black",
                                         size = font.size, hjust =1 ),
              axis.title = element_text(margin=margin(10, 5, 0, 0),
                                        color = "black",
                                        size = font.size),
              axis.title.y = element_text(angle=90)
              )
}

##' interal method for enrichment analysis
##'
##' using the hypergeometric model
##' @title enrich.internal
##' @param gene a vector of entrez gene id.
##' @param pvalueCutoff Cutoff value of pvalue.
##' @param pAdjustMethod one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"
##' @param universe background genes, default is the intersection of the 'universe' with genes that have annotations. 
##' Users can set `options(enrichment_force_universe = TRUE)` to force the 'universe' untouched.
##' @param minGSSize minimal size of genes annotated by Ontology term for testing.
##' @param maxGSSize maximal size of each geneSet for analyzing
##' @param qvalueCutoff cutoff of qvalue
##' @param USER_DATA ontology information
##' @return  A \code{enrichResult} instance.
##' @importClassesFrom methods data.frame
##' @importFrom qvalue qvalue
##' @importFrom methods new
##' @importFrom stats phyper
##' @importFrom stats p.adjust
##' @keywords manip
##' @author Guangchuang Yu \url{https://yulab-smu.top}
enricher_internal <- function(gene,
                              pvalueCutoff,
                              pAdjustMethod="BH",
                              universe = NULL,
                              minGSSize=10,
                              maxGSSize=500,
                              qvalueCutoff=0.2,
                              USER_DATA){

    ## query external ID to Term ID
    gene <- as.character(unique(gene))
    qExtID2TermID <- EXTID2TERMID(gene, USER_DATA)
    qTermID <- unlist(qExtID2TermID)
    if (is.null(qTermID)) {
        message("--> No gene can be mapped....")
        if (inherits(USER_DATA, "environment")) {
            p2e <- get("PATHID2EXTID", envir=USER_DATA)
            sg <- unique(unlist(p2e[1:10]))
        } else {
            sg <- unique(USER_DATA@gsid2gene$gene[1:100])
        }
        sg <- sample(sg, min(length(sg), 6))
        message("--> Expected input gene ID: ", paste0(sg, collapse=','))

        message("--> return NULL...")
        return(NULL)
    }

    ## Term ID -- query external ID association list.
    qExtID2TermID.df <- data.frame(extID=rep(names(qExtID2TermID),
                                             times=lapply(qExtID2TermID, length)),
                                   termID=qTermID)
    qExtID2TermID.df <- unique(qExtID2TermID.df)

    qTermID2ExtID <- with(qExtID2TermID.df,
                          split(as.character(extID), as.character(termID)))

    extID <- ALLEXTID(USER_DATA)
    if (missing(universe))
        universe <- NULL
    if(!is.null(universe)) {
        if (is.character(universe)) {
            force_universe <- getOption("enrichment_force_universe", FALSE)
            if (force_universe) {
                extID <- universe
            } else {
                extID <- intersect(extID, universe)
            }
        } else {
            ## https://github.com/YuLab-SMU/clusterProfiler/issues/217
            message("`universe` is not in character and will be ignored...")
        }
    }

    qTermID2ExtID <- lapply(qTermID2ExtID, intersect, extID)

    ## Term ID annotate query external ID
    qTermID <- unique(names(qTermID2ExtID))


    termID2ExtID <- TERMID2EXTID(qTermID, USER_DATA)
    termID2ExtID <- lapply(termID2ExtID, intersect, extID)

    geneSets <- termID2ExtID

    idx <- get_geneSet_index(termID2ExtID, minGSSize, maxGSSize)

    if (sum(idx) == 0) {
        msg <- paste("No gene sets have size between", minGSSize, "and", maxGSSize, "...")
        message(msg)
        message("--> return NULL...")
        return (NULL)
    }

    termID2ExtID <- termID2ExtID[idx]
    qTermID2ExtID <- qTermID2ExtID[idx]
    qTermID <- unique(names(qTermID2ExtID))

    ## prepare parameter for hypergeometric test
    k <- sapply(qTermID2ExtID, length)
    k <- k[qTermID]
    M <- sapply(termID2ExtID, length)
    M <- M[qTermID]

    N <- rep(length(extID), length(M))
    ## n <- rep(length(gene), length(M)) ## those genes that have no annotation should drop.
    n <- rep(length(qExtID2TermID), length(M))
    args.df <- data.frame(numWdrawn=k-1, ## White balls drawn
                          numW=M,        ## White balls
                          numB=N-M,      ## Black balls
                          numDrawn=n)    ## balls drawn


    ## calcute pvalues based on hypergeometric model
    pvalues <- apply(args.df, 1, function(n)
                     phyper(n[1], n[2], n[3], n[4], lower.tail=FALSE)
                     )

    ## gene ratio and background ratio
    #GeneRatio <- apply(data.frame(a=k, b=n), 1, function(x)
    #                   paste(x[1], "/", x[2], sep="", collapse="")
    #                   )
    #BgRatio <- apply(data.frame(a=M, b=N), 1, function(x)
    #                 paste(x[1], "/", x[2], sep="", collapse="")
    #                 )

    GeneRatio <- sprintf("%s/%s", k, n)
    BgRatio <- sprintf("%s/%s", M, N)
    RichFactor <- k / M
    FoldEnrichment <- RichFactor * N / n 

    # mu and sigma are the mean and standard deviation of the hypergeometric distribution
    ## https://en.wikipedia.org/wiki/Hypergeometric_distribution
    mu <- M * n / N
    sigma <- mu * (N - n) * (N - M) / N / (N-1)
    zScore <- (k - mu)/sqrt(sigma)
    Over <- data.frame(ID = as.character(qTermID),
                       GeneRatio = GeneRatio,
                       BgRatio = BgRatio,
                       RichFactor = RichFactor,
                       FoldEnrichment = FoldEnrichment,
                       zScore = zScore, 
                       pvalue = pvalues,
                       stringsAsFactors = FALSE)

    p.adj <- p.adjust(Over$pvalue, method=pAdjustMethod)
    qobj <- tryCatch(qvalue(p=Over$pvalue, lambda=0.05, pi0.method="bootstrap"), error=function(e) NULL)

    # if (class(qobj) == "qvalue") {
    if (inherits(qobj, "qvalue")) {
        qvalues <- qobj$qvalues
    } else {
        qvalues <- NA
    }

    geneID <- sapply(qTermID2ExtID, function(i) paste(i, collapse="/"))
    geneID <- geneID[qTermID]
    Over <- data.frame(Over,
                       p.adjust = p.adj,
                       qvalue = qvalues,
                       geneID = geneID,
                       Count = k,
                       stringsAsFactors = FALSE)

    Description <- TERM2NAME(qTermID, USER_DATA)

    if (length(qTermID) != length(Description)) {
        idx <- qTermID %in% names(Description)
        Over <- Over[idx,]
    }
    Over$Description <- Description
    nc <- ncol(Over)
    Over <- Over[, c(1,nc, 2:(nc-1))]


    Over <- Over[order(pvalues),]


    Over$ID <- as.character(Over$ID)
    Over$Description <- as.character(Over$Description)

    row.names(Over) <- as.character(Over$ID)

    x <- new("enrichResult",
             result         = Over,
             pvalueCutoff   = pvalueCutoff,
             pAdjustMethod  = pAdjustMethod,
             qvalueCutoff   = qvalueCutoff,
             gene           = as.character(gene),
             universe       = extID,
             geneSets       = geneSets,
             organism       = "UNKNOWN",
             keytype        = "UNKNOWN",
             ontology       = "UNKNOWN",
             readable       = FALSE
             )
    if (inherits(USER_DATA, "GSON")) {
        if (!is.null(USER_DATA@keytype)) {
            x@keytype <- USER_DATA@keytype
        }
        if (!is.null(USER_DATA@species)) {
            x@organism <- USER_DATA@species
        } 
        if (!is.null(USER_DATA@gsname)) {
            x@ontology <- gsub(".*;", "", USER_DATA@gsname)
        }
    }
    return (x)
}


get_enriched <- function(object) {

    Over <- object@result

    pvalueCutoff <- object@pvalueCutoff
    if (length(pvalueCutoff) != 0) {
        ## if groupGO result, numeric(0)
        Over <- Over[ Over$pvalue <= pvalueCutoff, ]
        Over <- Over[ Over$p.adjust <= pvalueCutoff, ]
    }

    qvalueCutoff <- object@qvalueCutoff
    if (length(qvalueCutoff) != 0) {
        if (! any(is.na(Over$qvalue))) {
            if (length(qvalueCutoff) > 0)
                Over <- Over[ Over$qvalue <= qvalueCutoff, ]
        }
    }

    object@result <- Over
    return(object)
}


EXTID2TERMID <- function(gene, USER_DATA) {
    if (inherits(USER_DATA, "environment")) { 
        EXTID2PATHID <- get("EXTID2PATHID", envir = USER_DATA)

        qExtID2Path <- EXTID2PATHID[gene]
    } else if (inherits(USER_DATA, "GSON")) {
        gsid2gene <- USER_DATA@gsid2gene
        qExtID2Path <- setNames(lapply(gene, function(x) {
            subset(gsid2gene, gsid2gene$gene == x)[["gsid"]]
        }), gene)
    } else {
        stop("not supported")
    }

    len <- sapply(qExtID2Path, length)
    notZero.idx <- len != 0
    qExtID2Path <- qExtID2Path[notZero.idx]

    return(qExtID2Path)
}

ALLEXTID <- function(USER_DATA) {
    if (inherits(USER_DATA, "environment")) { 
        PATHID2EXTID <- get("PATHID2EXTID", envir = USER_DATA)
        res <- unique(unlist(PATHID2EXTID))
    } else if (inherits(USER_DATA, "GSON")) {
        gsid2gene <- USER_DATA@gsid2gene
        res <- unique(gsid2gene$gene)
    } else {
        stop("not supported")
    }

    return(res)
}


TERMID2EXTID <- function(term, USER_DATA) {
    if (inherits(USER_DATA, "environment")) { 
        PATHID2EXTID <- get("PATHID2EXTID", envir = USER_DATA)
        res <- PATHID2EXTID[term]
    } else if (inherits(USER_DATA, "GSON")) {
        gsid2gene <- USER_DATA@gsid2gene
        res <- setNames(lapply(term, function(x) {
            subset(gsid2gene, gsid2gene$gsid == x)[["gene"]]
        }), term)
    } else {
        stop("not supported")
    }    

    return(res)
}

TERM2NAME <- function(term, USER_DATA) {
    if (inherits(USER_DATA, "environment")) { 
        PATHID2NAME <- get("PATHID2NAME", envir = USER_DATA)
        #if (is.null(PATHID2NAME) || is.na(PATHID2NAME)) {
        if (is.null(PATHID2NAME) || all(is.na(PATHID2NAME))) {
            return(as.character(term))
        }
        res <- PATHID2NAME[term]
        i <-  is.na(res)
        res[i] <- term[i]
    } else if (inherits(USER_DATA, "GSON")) {
        gsid2name <- USER_DATA@gsid2name
        i <- match(term, gsid2name$gsid)
        j <- !is.na(i)
        res <- term
        res[j] <- gsid2name$name[i[j]]
    } else {
        res <- as.character(term)
    }

    names(res) <- term
    return(res) 
}

get_geneSet_index <- function(geneSets, minGSSize, maxGSSize) {
    if (is.na(minGSSize) || is.null(minGSSize))
        minGSSize <- 1
    if (is.na(maxGSSize) || is.null(maxGSSize))
        maxGSSize <- Inf #.Machine$integer.max

    ## index of geneSets in used.
    ## logical
    geneSet_size <- sapply(geneSets, length)
    idx <-  minGSSize <= geneSet_size & geneSet_size <= maxGSSize
    return(idx)
}

##' Enrichment analysis based on the DisGeNET (\url{http://www.disgenet.org/})
##'
##' given a vector of genes, this function will return the enrichment NCG
##' categories with FDR control
##'
##'
##' @inheritParams enrichNCG
##' @return A \code{enrichResult} instance
##' @export
##' @references Janet et al. (2015) DisGeNET: a discovery platform for the dynamical exploration of human diseases and their genes. \emph{Database} bav028
##' \url{http://database.oxfordjournals.org/content/2015/bav028.long}
##' @author Guangchuang Yu
enrichDGN <- function(gene,
                      pvalueCutoff = 0.05,
                      pAdjustMethod = "BH",
                      universe,
                      minGSSize = 10,
                      maxGSSize = 500,
                      qvalueCutoff = 0.2,
                      readable = FALSE){

    enrichDisease(gene = gene,
                  pvalueCutoff = pvalueCutoff,
                  pAdjustMethod = pAdjustMethod,
                  universe = universe,
                  minGSSize = minGSSize,
                  maxGSSize = maxGSSize,
                  qvalueCutoff = qvalueCutoff,
                  readable = readable,
                  ontology = "DisGeNET")
    
}

get_DGN_data <- function() {
    if (!exists(".DOSEenv")) .initial()
    .DOSEEnv <- get(".DOSEEnv", envir = .GlobalEnv)
    
    if (!exists(".DGN_DOSE_Env", envir=.DOSEEnv)) {
        tryCatch(utils::data(list="DGN_EXTID2PATHID", package="DOSE"))
        tryCatch(utils::data(list="DGN_PATHID2EXTID", package="DOSE"))
        tryCatch(utils::data(list="DGN_PATHID2NAME", package="DOSE"))
        EXTID2PATHID <- DGN_EXTID2PATHID <- get("DGN_EXTID2PATHID")
        PATHID2EXTID <- DGN_PATHID2EXTID <- get("DGN_PATHID2EXTID")
        PATHID2NAME <- DGN_PATHID2NAME <- get("DGN_PATHID2NAME")

        rm(DGN_EXTID2PATHID, envir = .GlobalEnv)
        rm(DGN_PATHID2EXTID, envir = .GlobalEnv)
        rm(DGN_PATHID2NAME, envir = .GlobalEnv)

        assign(".DGN_DOSE_Env", new.env(), envir = .DOSEEnv)
        .DGN_DOSE_Env <- get(".DGN_DOSE_Env", envir = .DOSEEnv)
        assign("EXTID2PATHID", EXTID2PATHID, envir = .DGN_DOSE_Env)
        assign("PATHID2EXTID", PATHID2EXTID, envir = .DGN_DOSE_Env)
        assign("PATHID2NAME", PATHID2NAME, envir = .DGN_DOSE_Env)
    }
    get(".DGN_DOSE_Env", envir = .DOSEEnv)
}

##' measuring similarities between two DO term vectors.
##'
##' provide two term vectors, this function will calculate their similarities.
##' @title doseSim
##' @rdname doseSim
##' @param DOID1 DO term, MPO term or HPO term vector
##' @param DOID2 DO term, MPO term or HPO term vector
##' @param ont one of "HDO", "HPO" and "MPO"
##' @param measure one of "Wang", "Resnik", "Rel", "Jiang", "Lin", and "TCSS".
##' @return score matrix
##' @importFrom GOSemSim termSim
##' @export
##' @author Guangchuang Yu \url{https://yulab-smu.top}
doseSim <- function(DOID1,
                  DOID2,
                  measure="Wang",
                  ont = "HDO") {
    ont <- match.arg(ont, c("DO", "HDO", "MPO", "HPO"))                

    if (ont == "DO") ont <- 'HDO'

    processTCSS <- FALSE
    if (measure == "TCSS") {
        processTCSS <- TRUE
    } 

    scores <- GOSemSim::termSim(
        DOID1,
        DOID2, 
        semdata2(processTCSS = processTCSS, ont = ont), 
        measure
    )    

    if(length(scores) == 1)
        scores <- as.numeric(scores)

    return(scores)
}

##' @rdname doseSim
##' @export
doSim <- doseSim

##' DO Enrichment Analysis 
##'
##' Given a vector of genes, this function will return the enrichment DO
##' categories with FDR control.
##'
##' @rdname enrichDO
##' @param ont one of "HDO", "HPO" or "MPO".
##' @param organism one of "hsa" and "mmu"
##' @inheritParams enrichNCG
##' @return A \code{enrichResult} instance.
##' @export
##' @seealso \code{\link{enrichResult-class}}
##' @author Guangchuang Yu \url{https://yulab-smu.top}
##' @keywords manip
##' @examples
##'
##'	data(geneList)
##' 	gene = names(geneList)[geneList > 1]
##' 	yy = enrichDO(gene, pvalueCutoff=0.05)
##' 	summary(yy)
##'
enrichDO <- function(gene, ont="HDO",
                     organism = "hsa",
                     pvalueCutoff=0.05,
                     pAdjustMethod="BH",
                     universe,
                     minGSSize = 10,
                     maxGSSize = 500,
                     qvalueCutoff=0.2,
                     readable = FALSE){

    enrichDisease(gene = gene,
                  organism = organism,
                  pvalueCutoff = pvalueCutoff,
                  pAdjustMethod = pAdjustMethod,
                  universe = universe,
                  minGSSize = minGSSize,
                  maxGSSize = maxGSSize,
                  qvalueCutoff = qvalueCutoff,
                  readable = readable,
                  ontology = ont)
}

##' Pairwise semantic similarity for a list of gene clusters
##'
##'
##' @title mclusterSim
##' @param clusters A list of gene clusters
##' @param organism organism
##' @param ont one of "HDO", "HPO" and "MPO"
##' @param measure one of "Wang", "Resnik", "Rel", "Jiang", and "Lin".
##' @param combine One of "max", "avg", "rcmax", "BMA" methods, for combining semantic similarity scores of multiple DO terms associated with gene/protein.
##' @return similarity matrix
##' @importFrom GOSemSim combineScores
##' @export
##' @author Guangchuang Yu 
##' @examples
##' \dontrun{
##'	cluster1 <- c("835", "5261","241")
##'	cluster2 <- c("578","582")
##'	cluster3 <- c("307", "308", "317")
##'	clusters <- list(a=cluster1, b=cluster2, c=cluster3)
##'	mclusterSim(clusters, measure="Wang")
##' }
mclusterSim <- function(clusters, 
                        ont = "HDO",
                        organism = "hsa",
                        measure="Wang", 
                        combine="BMA") {
    if (ont == "DO") ont <- 'HDO'
                        
    cluster_dos <- list()
    for (i in seq_along(clusters)) {
        cluster_dos[[i]] <- unlist(sapply(clusters[[i]], gene2DO, organism = organism))
    }
    n <- length(clusters)
    scores <- matrix(NA, nrow=n, ncol=n)
    rownames(scores) <- names(clusters)
    colnames(scores) <- names(clusters)

    for (i in seq_along(cluster_dos)) {
        do1 <- cluster_dos[[i]]
        do1 <- do1[!is.na(do1)]
        for (j in 1:i) {
            do2 <- cluster_dos[[j]]
            do2 <- do2[!is.na(do2)]
            if (length(do1) != 0 && length(do2) != 0) {
                s <- doseSim(do1, do2, measure = measure, ont = ont)
                scores[i,j] <- combineScores(s, combine)
                if (i != j) {
                    scores[j, i] <- scores[i, j]
                }
            }
        }
    }
    removeRowNA <- apply(!is.na(scores), 1, sum)>0
    removeColNA <- apply(!is.na(scores), 2, sum)>0
    return(scores[removeRowNA, removeColNA])
}

#' geneID generic
#'
#' @param x enrichResult object
#' @return 'geneID' return the 'geneID' column of the enriched result which can be converted to data.frame via 'as.data.frame'
#' @export
#' @examples
#' data(geneList, package="DOSE")
#' de <- names(geneList)[1:100]
#' x <- enrichDO(de)
#' geneID(x)
geneID <- function(x) {
   UseMethod("geneID", x)
}

#' geneInCategory generic
#'
#' @param x enrichResult
#' @return 'geneInCategory' return a list of genes, by spliting the input gene vector to enriched functional categories
#' @export
#' @examples
#' data(geneList, package="DOSE")
#' de <- names(geneList)[1:100]
#' x <- enrichDO(de)
#' geneInCategory(x)
geneInCategory <- function(x) {
   UseMethod("geneInCategory", x)
}

##' filter enriched result by gene set size or gene count
##'
##' 
##' @title gsfilter
##' @param x instance of enrichResult or compareClusterResult
##' @param by one of 'GSSize' or 'Count'
##' @param min minimal size
##' @param max maximal size
##' @importFrom methods is
##' @return update object
##' @export
##' @author Guangchuang Yu
gsfilter <- function(x, by="GSSize", min=NA, max=NA) {
    by <- match.arg(by, c("GSSize", "Count"))
    if (is(x, "enrichResult")) {
        result <- x@result
    } else {
        result <- x@compareClusterResult
    }
    
    if (by == "GSSize") {
        n <- as.numeric(gsub("^(\\d+)/\\d+$",'\\1',  as.character(result$BgRatio)))
    } else {
        n <- result$Count
    }
    if (!is.na(min)) {
        min_lidx <- n >= min
    } else {
        min_lidx <- TRUE
    }
    
    if (!is.na(max)) {
        max_lidx <- n <= max
    } else {
        max_lidx <- TRUE
    }

    idx <- min_lidx & max_lidx

    if (is(x, "enrichResult")) {
        x@result <- result[idx,]
    } else {
        x@compareClusterResult <- result[idx,]
    }
    return(x)
}

# download from http://ncg.kcl.ac.uk/download.php
# NCG 6.0: All cancer genes -> •List of 2372 cancer genes and supporting literature
# NCG 7.0: List of all 3347 cancer drivers and their annotation and supporting evidence
#x=read.delim("NCG6_cancergenes.tsv", stringsAsFactor=F)
x=read.delim("NCG6_cancergenes.tsv", stringsAsFactor=F, encoding = "latin1")
path2gene <- x[, c("cancer_type", "entrez")]
path2gene <- path2gene[path2gene[,1] != '',]

## gene2name <- x[, c("entrez", "symbol")]

path2name=NULL

.NCG_DOSE_Env <- DOSE:::build_Anno(path2gene, path2name)

NCG_EXTID2PATHID = get("EXTID2PATHID", envir=.NCG_DOSE_Env)
NCG_PATHID2EXTID = get("PATHID2EXTID", envir = .NCG_DOSE_Env)
NCG_PATHID2NAME = get("PATHID2NAME", envir = .NCG_DOSE_Env)

save(NCG_EXTID2PATHID, file = "NCG_EXTID2PATHID.rda", compress='xz')
save(NCG_PATHID2EXTID, file="NCG_PATHID2EXTID.rda", compress='xz')
save(NCG_PATHID2NAME, file="NCG_PATHID2NAME.rda", compress='xz')

library(DOSE)

context("doSim")

test_that("doSim", {
    res <- sapply(c("Wang", "Lin", "Jiang", "Resnik", "Rel"), function(mm) {
        doSim("DOID:1002", "DOID:10003", measure=mm)
    })
    expect_true(all(res >= 0) && all(res <= 1))
})

---
title: "DOSE: Disease Ontology Semantic and Enrichment analysis"
author: 
- name: Guangchuang Yu
  email: guangchuangyu@gmail.com
  affiliation: Department of Bioinformatics, School of Basic Medical Sciences, Southern Medical University
date: "`r Sys.Date()`"
biblio-style: apalike
output:
  prettydoc::html_pretty:
    toc: true
    theme: cayman
    highlight: github
  pdf_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{DOSE}
  %\VignettePackage{DOSE}
  % \VignetteEngine{knitr::rmarkdown}
  % \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo=FALSE, results="asis", message=FALSE}
knitr::opts_chunk$set(tidy = FALSE,
                      warning = FALSE,
                      message = FALSE)

Biocpkg <- function (pkg) {
    sprintf("[%s](http://bioconductor.org/packages/%s)", pkg, pkg)
}

```


```{r echo=FALSE, results='hide', message=FALSE}
library(DOSE)
```


# Vignette

Please go to <https://yulab-smu.top/biomedical-knowledge-mining-book/> for the full vignette.



# Citation

If you use `r Biocpkg("DOSE")` in published research, please cite G. Yu (2015).


__*G Yu*__, LG Wang, GR Yan, QY He. DOSE: an R/Bioconductor package for Disease Ontology Semantic and Enrichment analysis. __*Bioinformatics*__ 2015, 31(4):608-609. <http://dx.doi.org/10.1093/bioinformatics/btu684>.


# Need helps?


If you have questions/issues, please post
to [Bioconductor support site](https://support.bioconductor.org/) and tag your
post with *DOSE*.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enricher_internal.R
\name{enricher_internal}
\alias{enricher_internal}
\title{enrich.internal}
\usage{
enricher_internal(
  gene,
  pvalueCutoff,
  pAdjustMethod = "BH",
  universe = NULL,
  minGSSize = 10,
  maxGSSize = 500,
  qvalueCutoff = 0.2,
  USER_DATA
)
}
\arguments{
\item{gene}{a vector of entrez gene id.}

\item{pvalueCutoff}{Cutoff value of pvalue.}

\item{pAdjustMethod}{one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"}

\item{universe}{background genes, default is the intersection of the 'universe' with genes that have annotations. 
Users can set `options(enrichment_force_universe = TRUE)` to force the 'universe' untouched.}

\item{minGSSize}{minimal size of genes annotated by Ontology term for testing.}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{qvalueCutoff}{cutoff of qvalue}

\item{USER_DATA}{ontology information}
}
\value{
A \code{enrichResult} instance.
}
\description{
interal method for enrichment analysis
}
\details{
using the hypergeometric model
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doSim.R
\name{doseSim}
\alias{doseSim}
\alias{doSim}
\title{doseSim}
\usage{
doseSim(DOID1, DOID2, measure = "Wang", ont = "HDO")

doSim(DOID1, DOID2, measure = "Wang", ont = "HDO")
}
\arguments{
\item{DOID1}{DO term, MPO term or HPO term vector}

\item{DOID2}{DO term, MPO term or HPO term vector}

\item{measure}{one of "Wang", "Resnik", "Rel", "Jiang", "Lin", and "TCSS".}

\item{ont}{one of "HDO", "HPO" and "MPO"}
}
\value{
score matrix
}
\description{
measuring similarities between two DO term vectors.
}
\details{
provide two term vectors, this function will calculate their similarities.
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mclusterSim.R
\name{mclusterSim}
\alias{mclusterSim}
\title{mclusterSim}
\usage{
mclusterSim(
  clusters,
  ont = "HDO",
  organism = "hsa",
  measure = "Wang",
  combine = "BMA"
)
}
\arguments{
\item{clusters}{A list of gene clusters}

\item{ont}{one of "HDO", "HPO" and "MPO"}

\item{organism}{organism}

\item{measure}{one of "Wang", "Resnik", "Rel", "Jiang", and "Lin".}

\item{combine}{One of "max", "avg", "rcmax", "BMA" methods, for combining semantic similarity scores of multiple DO terms associated with gene/protein.}
}
\value{
similarity matrix
}
\description{
Pairwise semantic similarity for a list of gene clusters
}
\examples{
\dontrun{
cluster1 <- c("835", "5261","241")
cluster2 <- c("578","582")
cluster3 <- c("307", "308", "317")
clusters <- list(a=cluster1, b=cluster2, c=cluster3)
mclusterSim(clusters, measure="Wang")
}
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_ratio.R
\name{parse_ratio}
\alias{parse_ratio}
\title{parse_ratio}
\usage{
parse_ratio(ratio)
}
\arguments{
\item{ratio}{character vector of ratio to parse}
}
\value{
A numeric vector (double) of parsed ratio
}
\description{
parse character ratio to double value, such as 1/5 to 0.2
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DOSE-package.R
\docType{data}
\name{DataSet}
\alias{DataSet}
\alias{geneList}
\alias{NCG_EXTID2PATHID}
\alias{NCG_PATHID2EXTID}
\alias{NCG_PATHID2NAME}
\alias{DGN_EXTID2PATHID}
\alias{DGN_PATHID2EXTID}
\alias{DGN_PATHID2NAME}
\alias{VDGN_EXTID2PATHID}
\alias{VDGN_PATHID2EXTID}
\alias{VDGN_PATHID2NAME}
\title{Datasets}
\description{
Information content and DO term to entrez gene IDs mapping
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DOSE-package.R
\docType{package}
\name{DOSE-package}
\alias{DOSE}
\alias{DOSE-package}
\title{DOSE: Disease Ontology Semantic and Enrichment analysis}
\description{
This package implements five methods proposed by Resnik, Schlicker, Jiang, Lin and Wang respectively for measuring semantic similarities among DO terms and gene products. Enrichment analyses including hypergeometric model and gene set enrichment analysis are also implemented for discovering disease associations of high-throughput biological data.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://yulab-smu.top/biomedical-knowledge-mining-book/}
  \item Report bugs at \url{https://github.com/GuangchuangYu/DOSE/issues}
}

}
\author{
\strong{Maintainer}: Guangchuang Yu \email{guangchuangyu@gmail.com}

Other contributors:
\itemize{
  \item Li-Gen Wang \email{reeganwang020@gmail.com} [contributor]
  \item Vladislav Petyuk \email{petyuk@gmail.com} [contributor]
  \item Giovanni Dall'Olio \email{giovanni.dallolio@upf.edu} [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneSim.R
\name{geneSim}
\alias{geneSim}
\title{geneSim}
\usage{
geneSim(
  geneID1,
  geneID2 = NULL,
  ont = "HDO",
  organism = "hsa",
  measure = "Wang",
  combine = "BMA"
)
}
\arguments{
\item{geneID1}{entrez gene vector}

\item{geneID2}{entrez gene vector}

\item{ont}{one of "HDO" and "MPO"}

\item{organism}{one of "hsa" and "mmu"}

\item{measure}{one of "Wang", "Resnik", "Rel", "Jiang", and "Lin".}

\item{combine}{One of "max", "avg", "rcmax", "BMA" methods, for combining semantic similarity scores of multiple DO terms associated with gene/protein.}
}
\value{
score matrix
}
\description{
measuring similarities bewteen two gene vectors.
}
\details{
provide two entrez gene vectors, this function will calculate their similarity.
}
\examples{
g <- c("835", "5261","241", "994")
geneSim(g)
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichNCG.R
\name{enrichNCG}
\alias{enrichNCG}
\title{enrichNCG}
\usage{
enrichNCG(
  gene,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  universe,
  minGSSize = 10,
  maxGSSize = 500,
  qvalueCutoff = 0.2,
  readable = FALSE
)
}
\arguments{
\item{gene}{a vector of entrez gene id}

\item{pvalueCutoff}{pvalue cutoff}

\item{pAdjustMethod}{one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"}

\item{universe}{background genes}

\item{minGSSize}{minimal size of genes annotated by NCG category for testing}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{qvalueCutoff}{qvalue cutoff}

\item{readable}{whether mapping gene ID to gene Name}
}
\value{
A \code{enrichResult} instance
}
\description{
Enrichment analysis based on the Network of Cancer Genes database (http://ncg.kcl.ac.uk/)
}
\details{
given a vector of genes, this function will return the enrichment NCG
categories with FDR control
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{EXTID2NAME}
\alias{EXTID2NAME}
\title{EXTID2NAME}
\usage{
EXTID2NAME(OrgDb, geneID, keytype)
}
\arguments{
\item{OrgDb}{OrgDb}

\item{geneID}{entrez gene ID}

\item{keytype}{keytype}
}
\value{
gene symbol
}
\description{
mapping gene ID to gene Symbol
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessor.R
\docType{methods}
\name{summary}
\alias{summary}
\title{summary method}
\usage{
summary(object, ...)

summary(object, ...)
}
\arguments{
\item{object}{A \code{enrichResult} instance.}

\item{...}{additional parameter}
}
\value{
A data frame

A data frame
}
\description{
summary method for \code{gseaResult} instance

summary method for \code{enrichResult} instance
}
\author{
Guangchuang Yu \url{https://guangchuangyu.github.io}

Guangchuang Yu \url{http://guangchuangyu.github.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterSim.R
\name{clusterSim}
\alias{clusterSim}
\title{clusterSim}
\usage{
clusterSim(
  cluster1,
  cluster2,
  ont = "HDO",
  organism = "hsa",
  measure = "Wang",
  combine = "BMA"
)
}
\arguments{
\item{cluster1}{a vector of gene IDs}

\item{cluster2}{another vector of gene IDs}

\item{ont}{one of "HDO", "HPO" and "MPO"}

\item{organism}{one of "hsa" and "mmu"}

\item{measure}{One of "Resnik", "Lin", "Rel", "Jiang" and "Wang" methods.}

\item{combine}{One of "max", "avg", "rcmax", "BMA" methods, for combining}
}
\value{
similarity
}
\description{
semantic similarity between two gene clusters
}
\details{
given two gene clusters, this function calculates semantic similarity between them.
}
\examples{
\dontrun{
cluster1 <- c("835", "5261","241", "994")
cluster2 <- c("307", "308", "317", "321", "506", "540", "378", "388", "396")
clusterSim(cluster1, cluster2)
}
}
\author{
Yu Guangchuang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00-AllClasses.R
\docType{class}
\name{gseaResult-class}
\alias{gseaResult-class}
\alias{gseahResult-class}
\alias{show,gseaResult-method}
\alias{summary,gseaResult-method}
\title{Class "gseaResult"
This class represents the result of GSEA analysis}
\description{
Class "gseaResult"
This class represents the result of GSEA analysis
}
\section{Slots}{

\describe{
\item{\code{result}}{GSEA anaysis}

\item{\code{organism}}{organism}

\item{\code{setType}}{setType}

\item{\code{geneSets}}{geneSets}

\item{\code{geneList}}{order rank geneList}

\item{\code{keytype}}{ID type of gene}

\item{\code{permScores}}{permutation scores}

\item{\code{params}}{parameters}

\item{\code{gene2Symbol}}{gene ID to Symbol}

\item{\code{readable}}{whether convert gene ID to symbol}

\item{\code{dr}}{dimension reduction result}
}}

\author{
Guangchuang Yu \url{https://yulab-smu.top}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gseAnalyzer.R
\name{gseNCG}
\alias{gseNCG}
\title{NCG Gene Set Enrichment Analysis}
\usage{
gseNCG(
  geneList,
  exponent = 1,
  minGSSize = 10,
  maxGSSize = 500,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  verbose = TRUE,
  seed = FALSE,
  by = "fgsea",
  ...
)
}
\arguments{
\item{geneList}{order ranked geneList}

\item{exponent}{weight of each step}

\item{minGSSize}{minimal size of each geneSet for analyzing}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{pvalueCutoff}{pvalue Cutoff}

\item{pAdjustMethod}{p value adjustment method}

\item{verbose}{print message or not}

\item{seed}{logical}

\item{by}{one of 'fgsea' or 'DOSE'}

\item{...}{other parameter}
}
\value{
gseaResult object
}
\description{
perform gsea analysis
}
\author{
Yu Guangchuang
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{gene2DO}
\alias{gene2DO}
\title{convert Gene ID to DO Terms}
\usage{
gene2DO(gene, organism = "hsa", ont = "HDO")
}
\arguments{
\item{gene}{entrez gene ID}

\item{organism}{organism}

\item{ont}{ont}
}
\value{
DO Terms
}
\description{
provide gene ID, this function will convert to the corresponding DO Terms
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{computeIC}
\alias{computeIC}
\title{compute information content}
\usage{
computeIC(ont = "HDO")
}
\arguments{
\item{ont}{one of "DO", "HPO" and "MPO"}
}
\description{
compute information content
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplot.R
\name{theme_dose}
\alias{theme_dose}
\title{theme_dose}
\usage{
theme_dose(font.size = 14)
}
\arguments{
\item{font.size}{font size}
}
\value{
ggplot theme
}
\description{
ggplot theme of DOSE
}
\examples{
library(ggplot2)
qplot(1:10) + theme_dose()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichDGN.R
\name{enrichDGN}
\alias{enrichDGN}
\title{Enrichment analysis based on the DisGeNET (\url{http://www.disgenet.org/})}
\usage{
enrichDGN(
  gene,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  universe,
  minGSSize = 10,
  maxGSSize = 500,
  qvalueCutoff = 0.2,
  readable = FALSE
)
}
\arguments{
\item{gene}{a vector of entrez gene id}

\item{pvalueCutoff}{pvalue cutoff}

\item{pAdjustMethod}{one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"}

\item{universe}{background genes}

\item{minGSSize}{minimal size of genes annotated by NCG category for testing}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{qvalueCutoff}{qvalue cutoff}

\item{readable}{whether mapping gene ID to gene Name}
}
\value{
A \code{enrichResult} instance
}
\description{
given a vector of genes, this function will return the enrichment NCG
categories with FDR control
}
\references{
Janet et al. (2015) DisGeNET: a discovery platform for the dynamical exploration of human diseases and their genes. \emph{Database} bav028
\url{http://database.oxfordjournals.org/content/2015/bav028.long}
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichDO.R
\name{enrichDO}
\alias{enrichDO}
\title{DO Enrichment Analysis}
\usage{
enrichDO(
  gene,
  ont = "HDO",
  organism = "hsa",
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  universe,
  minGSSize = 10,
  maxGSSize = 500,
  qvalueCutoff = 0.2,
  readable = FALSE
)
}
\arguments{
\item{gene}{a vector of entrez gene id}

\item{ont}{one of "HDO", "HPO" or "MPO".}

\item{organism}{one of "hsa" and "mmu"}

\item{pvalueCutoff}{pvalue cutoff}

\item{pAdjustMethod}{one of "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"}

\item{universe}{background genes}

\item{minGSSize}{minimal size of genes annotated by NCG category for testing}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{qvalueCutoff}{qvalue cutoff}

\item{readable}{whether mapping gene ID to gene Name}
}
\value{
A \code{enrichResult} instance.
}
\description{
Given a vector of genes, this function will return the enrichment DO
categories with FDR control.
}
\examples{

data(geneList)
	gene = names(geneList)[geneList > 1]
	yy = enrichDO(gene, pvalueCutoff=0.05)
	summary(yy)

}
\seealso{
\code{\link{enrichResult-class}}
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gseAnalyzer.R
\name{gseDO}
\alias{gseDO}
\title{DO Gene Set Enrichment Analysis}
\usage{
gseDO(
  geneList,
  ont = "HDO",
  organism = "hsa",
  exponent = 1,
  minGSSize = 10,
  maxGSSize = 500,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  verbose = TRUE,
  seed = FALSE,
  by = "fgsea",
  ...
)
}
\arguments{
\item{geneList}{order ranked geneList}

\item{ont}{one of "HDO", "HPO" or "MPO"}

\item{organism}{one of "hsa" and "mmu"}

\item{exponent}{weight of each step}

\item{minGSSize}{minimal size of each geneSet for analyzing}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{pvalueCutoff}{pvalue Cutoff}

\item{pAdjustMethod}{p value adjustment method}

\item{verbose}{print message or not}

\item{seed}{logical}

\item{by}{one of 'fgsea' or 'DOSE'}

\item{...}{other parameter}
}
\value{
gseaResult object
}
\description{
perform gsea analysis
}
\author{
Yu Guangchuang
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplot.R
\name{simplot}
\alias{simplot}
\title{simplot}
\usage{
simplot(
  sim,
  xlab = "",
  ylab = "",
  color.low = "white",
  color.high = "red",
  labs = TRUE,
  digits = 2,
  labs.size = 3,
  font.size = 14
)
}
\arguments{
\item{sim}{similarity matrix}

\item{xlab}{xlab}

\item{ylab}{ylab}

\item{color.low}{color of low value}

\item{color.high}{color of high value}

\item{labs}{logical, add text label or not}

\item{digits}{round digit numbers}

\item{labs.size}{lable size}

\item{font.size}{font size}
}
\value{
ggplot object
}
\description{
plotting similarity matrix
}
\author{
Yu Guangchuang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{geneInCategory}
\alias{geneInCategory}
\title{geneInCategory generic}
\usage{
geneInCategory(x)
}
\arguments{
\item{x}{enrichResult}
}
\value{
'geneInCategory' return a list of genes, by spliting the input gene vector to enriched functional categories
}
\description{
geneInCategory generic
}
\examples{
data(geneList, package="DOSE")
de <- names(geneList)[1:100]
x <- enrichDO(de)
geneInCategory(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00-AllClasses.R
\docType{class}
\name{enrichResult-class}
\alias{enrichResult-class}
\alias{show,enrichResult-method}
\alias{summary,enrichResult-method}
\title{Class "enrichResult"
This class represents the result of enrichment analysis.}
\description{
Class "enrichResult"
This class represents the result of enrichment analysis.
}
\section{Slots}{

\describe{
\item{\code{result}}{enrichment analysis}

\item{\code{pvalueCutoff}}{pvalueCutoff}

\item{\code{pAdjustMethod}}{pvalue adjust method}

\item{\code{qvalueCutoff}}{qvalueCutoff}

\item{\code{organism}}{only "human" supported}

\item{\code{ontology}}{biological ontology}

\item{\code{gene}}{Gene IDs}

\item{\code{keytype}}{Gene ID type}

\item{\code{universe}}{background gene}

\item{\code{gene2Symbol}}{mapping gene to Symbol}

\item{\code{geneSets}}{gene sets}

\item{\code{readable}}{logical flag of gene ID in symbol or not.}

\item{\code{termsim}}{Similarity between term}

\item{\code{method}}{method of calculating the similarity between nodes}

\item{\code{dr}}{dimension reduction result}
}}

\seealso{
\code{\link{enrichDO}}
}
\author{
Guangchuang Yu \url{https://yulab-smu.top}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{facet_grid}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{ggplot2}{\code{\link[ggplot2]{facet_grid}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{geneID}
\alias{geneID}
\title{geneID generic}
\usage{
geneID(x)
}
\arguments{
\item{x}{enrichResult object}
}
\value{
'geneID' return the 'geneID' column of the enriched result which can be converted to data.frame via 'as.data.frame'
}
\description{
geneID generic
}
\examples{
data(geneList, package="DOSE")
de <- names(geneList)[1:100]
x <- enrichDO(de)
geneID(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{GSEA_internal}
\alias{GSEA_internal}
\title{GSEA_internal}
\usage{
GSEA_internal(
  geneList,
  exponent,
  minGSSize,
  maxGSSize,
  eps,
  pvalueCutoff,
  pAdjustMethod,
  verbose,
  seed = FALSE,
  USER_DATA,
  by = "fgsea",
  ...
)
}
\arguments{
\item{geneList}{order ranked geneList}

\item{exponent}{weight of each step}

\item{minGSSize}{minimal size of each geneSet for analyzing}

\item{maxGSSize}{maximal size of each geneSet for analyzing}

\item{eps}{This parameter sets the boundary for calculating the p value.}

\item{pvalueCutoff}{p value Cutoff}

\item{pAdjustMethod}{p value adjustment method}

\item{verbose}{print message or not}

\item{seed}{set seed inside the function to make result reproducible. FALSE by default.}

\item{USER_DATA}{annotation data}

\item{by}{one of 'fgsea' or 'DOSE'}

\item{...}{other parameter}
}
\value{
gseaResult object
}
\description{
generic function for gene set enrichment analysis
}
\author{
Yu Guangchuang
}
